from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.avoid_post import AvoidPost
    from ..models.exclude_post import ExcludePost
    from ..models.max_speed_on_segment_post_inner import MaxSpeedOnSegmentPostInner


T = TypeVar("T", bound="GetRoutesByHandlePostParameters")


@_attrs_define
class GetRoutesByHandlePostParameters:
    """Parameters of the POST body for get route by handle

    Attributes:
        avoid (AvoidPost | Unset): Avoid routes that violate certain features of road network or that go through
            user-specified geographical bounding boxes.

            For the general description of the functionality please refer to the `avoid` parameter of the
            query string.

            Passing parameters in the POST body is suggested when the length of the parameters exceeds the
            limitation of the GET request.
        exclude (ExcludePost | Unset): User-specified properties that need to be strictly excluded during route
            calculation.

            For the general description of the functionality please refer to the `exclude` parameter of the
            query string.

            Passing parameters in the POST body is suggested when the length of the parameters exceeds the
            limitation of the GET request.
        max_speed_on_segment (list[MaxSpeedOnSegmentPostInner] | Unset): Segments with restrictions on maximum
            `baseSpeed`.

            For the general description of the functionality please refer to the `maxSpeedOnSegment` parameter of the
            query string.

            Passing parameters in the POST body is suggested when the length of the parameters exceeds the
            limitation of the GET request.

            Example of a parameter value excluding two segments:
            ```
            [
              {
                "segment": "here:cm:segment:207551710#+",
                "speed": 10
              },
              {
                "segment": "here:cm:segment:76771992",
                "speed": 1
              }
            ]
            ```

            **Notes**:
            - Maximum number of penalized segments in one request cannot be greater than 1000.
              "penalized segments" refer to segments that have a restrictions on maximum baseSpeed with `maxSpeedOnSegment`
              or avoided with `avoid[segments]`.
            - In case the same segment is penalized multiple times through values provided in the query string and/or the
            POST body,
              then the most restrictive value will be applied.
    """

    avoid: AvoidPost | Unset = UNSET
    exclude: ExcludePost | Unset = UNSET
    max_speed_on_segment: list[MaxSpeedOnSegmentPostInner] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        avoid: dict[str, Any] | Unset = UNSET
        if not isinstance(self.avoid, Unset):
            avoid = self.avoid.to_dict()

        exclude: dict[str, Any] | Unset = UNSET
        if not isinstance(self.exclude, Unset):
            exclude = self.exclude.to_dict()

        max_speed_on_segment: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.max_speed_on_segment, Unset):
            max_speed_on_segment = []
            for (
                componentsschemas_max_speed_on_segment_post_item_data
            ) in self.max_speed_on_segment:
                componentsschemas_max_speed_on_segment_post_item = (
                    componentsschemas_max_speed_on_segment_post_item_data.to_dict()
                )
                max_speed_on_segment.append(
                    componentsschemas_max_speed_on_segment_post_item
                )

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if avoid is not UNSET:
            field_dict["avoid"] = avoid
        if exclude is not UNSET:
            field_dict["exclude"] = exclude
        if max_speed_on_segment is not UNSET:
            field_dict["maxSpeedOnSegment"] = max_speed_on_segment

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.avoid_post import AvoidPost
        from ..models.exclude_post import ExcludePost
        from ..models.max_speed_on_segment_post_inner import MaxSpeedOnSegmentPostInner

        d = dict(src_dict)
        _avoid = d.pop("avoid", UNSET)
        avoid: AvoidPost | Unset
        if isinstance(_avoid, Unset):
            avoid = UNSET
        else:
            avoid = AvoidPost.from_dict(_avoid)

        _exclude = d.pop("exclude", UNSET)
        exclude: ExcludePost | Unset
        if isinstance(_exclude, Unset):
            exclude = UNSET
        else:
            exclude = ExcludePost.from_dict(_exclude)

        _max_speed_on_segment = d.pop("maxSpeedOnSegment", UNSET)
        max_speed_on_segment: list[MaxSpeedOnSegmentPostInner] | Unset = UNSET
        if _max_speed_on_segment is not UNSET:
            max_speed_on_segment = []
            for (
                componentsschemas_max_speed_on_segment_post_item_data
            ) in _max_speed_on_segment:
                componentsschemas_max_speed_on_segment_post_item = (
                    MaxSpeedOnSegmentPostInner.from_dict(
                        componentsschemas_max_speed_on_segment_post_item_data
                    )
                )

                max_speed_on_segment.append(
                    componentsschemas_max_speed_on_segment_post_item
                )

        get_routes_by_handle_post_parameters = cls(
            avoid=avoid,
            exclude=exclude,
            max_speed_on_segment=max_speed_on_segment,
        )

        get_routes_by_handle_post_parameters.additional_properties = d
        return get_routes_by_handle_post_parameters

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="Range")


@_attrs_define
class Range:
    """Ranges specified in terms of distance, travel time or energy consumption.

    Attributes:
        type_ (str): Specifies the type of range.

            Possible Values:
            - `distance` with units in meters with maximum value 1000000
            - `time` with units in seconds with maximum value 32400
            - `consumption` For fuel-based Vehicles : Static maximum value of 40000 with units in milliliters(ml) or
            grams(gms)
                            For EV : Maximum value of 650000 with units in Watt-hours (Wh). Dynamic limit up to the maximum
            value that allows the vehicle to travel a maximum of 900kms.
             Example: distance.
        values (str): A comma-separated list of ranges. The unit is defined by the `type` parameter. Example: 1000,2000.
    """

    type_: str
    values: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        values = self.values

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "values": values,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = d.pop("type")

        values = d.pop("values")

        range_ = cls(
            type_=type_,
            values=values,
        )

        range_.additional_properties = d
        return range_

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .access import Access
from .access_restriction_attributes import AccessRestrictionAttributes
from .address import Address
from .address_highlighting_information import AddressHighlightingInformation
from .address_usage import AddressUsage
from .address_usage_usage_type import AddressUsageUsageType
from .admin_id_section import AdminIdSection
from .admin_names import AdminNames
from .admin_names_preference import AdminNamesPreference
from .autocomplete_result_item import AutocompleteResultItem
from .autocomplete_result_item_administrative_area_type import (
    AutocompleteResultItemAdministrativeAreaType,
)
from .autocomplete_result_item_house_number_type import (
    AutocompleteResultItemHouseNumberType,
)
from .autocomplete_result_item_locality_type import AutocompleteResultItemLocalityType
from .autocomplete_result_item_result_type import AutocompleteResultItemResultType
from .autosuggest_body import AutosuggestBody
from .autosuggest_entity_result_item import AutosuggestEntityResultItem
from .autosuggest_entity_result_item_address_block_type import (
    AutosuggestEntityResultItemAddressBlockType,
)
from .autosuggest_entity_result_item_administrative_area_type import (
    AutosuggestEntityResultItemAdministrativeAreaType,
)
from .autosuggest_entity_result_item_house_number_type import (
    AutosuggestEntityResultItemHouseNumberType,
)
from .autosuggest_entity_result_item_locality_type import (
    AutosuggestEntityResultItemLocalityType,
)
from .autosuggest_entity_result_item_result_type import (
    AutosuggestEntityResultItemResultType,
)
from .autosuggest_query_result_item import AutosuggestQueryResultItem
from .autosuggest_query_result_item_result_type import (
    AutosuggestQueryResultItemResultType,
)
from .basic_access_point import BasicAccessPoint
from .browse_body import BrowseBody
from .browse_result_item import BrowseResultItem
from .browse_result_item_address_block_type import BrowseResultItemAddressBlockType
from .browse_result_item_administrative_area_type import (
    BrowseResultItemAdministrativeAreaType,
)
from .browse_result_item_locality_type import BrowseResultItemLocalityType
from .browse_result_item_result_type import BrowseResultItemResultType
from .car_fuel import CarFuel
from .car_fuel_type import CarFuelType
from .category import Category
from .category_ref import CategoryRef
from .chain import Chain
from .cm_version_section import CmVersionSection
from .contact import Contact
from .contact_information import ContactInformation
from .country_info import CountryInfo
from .discover_body import DiscoverBody
from .display_response_coordinate import DisplayResponseCoordinate
from .e_mobility_service_provider import EMobilityServiceProvider
from .editorial import Editorial
from .editorial_media_collection import EditorialMediaCollection
from .error_response import ErrorResponse
from .ev_availability_attributes import EvAvailabilityAttributes
from .ev_availability_connector import EvAvailabilityConnector
from .ev_availability_evse import EvAvailabilityEvse
from .ev_availability_evse_state import EvAvailabilityEvseState
from .ev_availability_station import EvAvailabilityStation
from .ev_charging_attributes import EvChargingAttributes
from .ev_charging_attributes_access import EvChargingAttributesAccess
from .ev_charging_point import EvChargingPoint
from .ev_connector import EvConnector
from .ev_name_id import EvNameId
from .ev_payment_support import EvPaymentSupport
from .ev_payment_support_id import EvPaymentSupportId
from .ev_station import EvStation
from .ev_station_connector_type_ids_item import EvStationConnectorTypeIdsItem
from .ev_station_current import EvStationCurrent
from .ev_station_payment_method_ids_item import EvStationPaymentMethodIdsItem
from .extended_access_point import ExtendedAccessPoint
from .extended_access_point_type import ExtendedAccessPointType
from .extended_attribute import ExtendedAttribute
from .field_score import FieldScore
from .fuel_additive import FuelAdditive
from .fuel_additive_type import FuelAdditiveType
from .fuel_price import FuelPrice
from .fuel_station import FuelStation
from .fuel_station_attributes import FuelStationAttributes
from .fuel_station_fuel_types_item import FuelStationFuelTypesItem
from .fuel_station_minimum_truck_class import FuelStationMinimumTruckClass
from .functional_class import FunctionalClass
from .functional_class_value import FunctionalClassValue
from .geocode_result_item import GeocodeResultItem
from .geocode_result_item_address_block_type import GeocodeResultItemAddressBlockType
from .geocode_result_item_administrative_area_type import (
    GeocodeResultItemAdministrativeAreaType,
)
from .geocode_result_item_house_number_type import GeocodeResultItemHouseNumberType
from .geocode_result_item_locality_type import GeocodeResultItemLocalityType
from .geocode_result_item_result_type import GeocodeResultItemResultType
from .get_autocomplete_postal_code_mode import GetAutocompletePostalCodeMode
from .get_autocomplete_show_item import GetAutocompleteShowItem
from .get_autocomplete_types_item import GetAutocompleteTypesItem
from .get_autosuggest_mobility_mode import GetAutosuggestMobilityMode
from .get_autosuggest_ranking import GetAutosuggestRanking
from .get_autosuggest_show_item import GetAutosuggestShowItem
from .get_autosuggest_show_map_references_item import (
    GetAutosuggestShowMapReferencesItem,
)
from .get_autosuggest_with_item import GetAutosuggestWithItem
from .get_browse_ranking import GetBrowseRanking
from .get_browse_show_item import GetBrowseShowItem
from .get_discover_mobility_mode import GetDiscoverMobilityMode
from .get_discover_ranking import GetDiscoverRanking
from .get_discover_show_item import GetDiscoverShowItem
from .get_discover_with_item import GetDiscoverWithItem
from .get_geocode_address_names_mode import GetGeocodeAddressNamesMode
from .get_geocode_postal_code_mode import GetGeocodePostalCodeMode
from .get_geocode_show_item import GetGeocodeShowItem
from .get_geocode_show_map_references_item import GetGeocodeShowMapReferencesItem
from .get_geocode_show_nav_attributes_item import GetGeocodeShowNavAttributesItem
from .get_geocode_show_related_item import GetGeocodeShowRelatedItem
from .get_geocode_show_translations_item import GetGeocodeShowTranslationsItem
from .get_geocode_types_item import GetGeocodeTypesItem
from .get_geocode_with_item import GetGeocodeWithItem
from .get_lookup_show_item import GetLookupShowItem
from .get_lookup_show_map_references_item import GetLookupShowMapReferencesItem
from .get_lookup_show_nav_attributes_item import GetLookupShowNavAttributesItem
from .get_lookup_show_related_item import GetLookupShowRelatedItem
from .get_revgeocode_show_item import GetRevgeocodeShowItem
from .get_revgeocode_show_map_references_item import GetRevgeocodeShowMapReferencesItem
from .get_revgeocode_show_nav_attributes_item import GetRevgeocodeShowNavAttributesItem
from .get_revgeocode_show_related_item import GetRevgeocodeShowRelatedItem
from .get_revgeocode_types_item import GetRevgeocodeTypesItem
from .get_revgeocode_with_item import GetRevgeocodeWithItem
from .image_media_collection import ImageMediaCollection
from .link_info_section import LinkInfoSection
from .lookup_response import LookupResponse
from .lookup_response_address_block_type import LookupResponseAddressBlockType
from .lookup_response_administrative_area_type import (
    LookupResponseAdministrativeAreaType,
)
from .lookup_response_closed_permanently import LookupResponseClosedPermanently
from .lookup_response_house_number_type import LookupResponseHouseNumberType
from .lookup_response_locality_type import LookupResponseLocalityType
from .lookup_response_result_type import LookupResponseResultType
from .map_reference_section import MapReferenceSection
from .map_reference_section_as import MapReferenceSectionAS
from .map_view import MapView
from .match_info import MatchInfo
from .match_info_qq import MatchInfoQq
from .media import Media
from .micro_point_address_section import MicroPointAddressSection
from .multi_result_error_section import MultiResultErrorSection
from .name import Name
from .name_type import NameType
from .navigation_attributes import NavigationAttributes
from .onebox_search_result_item import OneboxSearchResultItem
from .onebox_search_result_item_address_block_type import (
    OneboxSearchResultItemAddressBlockType,
)
from .onebox_search_result_item_administrative_area_type import (
    OneboxSearchResultItemAdministrativeAreaType,
)
from .onebox_search_result_item_house_number_type import (
    OneboxSearchResultItemHouseNumberType,
)
from .onebox_search_result_item_locality_type import OneboxSearchResultItemLocalityType
from .onebox_search_result_item_result_type import OneboxSearchResultItemResultType
from .open_search_autocomplete_response import OpenSearchAutocompleteResponse
from .open_search_autosuggest_response import OpenSearchAutosuggestResponse
from .open_search_browse_response import OpenSearchBrowseResponse
from .open_search_geocode_response import OpenSearchGeocodeResponse
from .open_search_multi_reverse_geocode_error_result import (
    OpenSearchMultiReverseGeocodeErrorResult,
)
from .open_search_multi_reverse_geocode_response import (
    OpenSearchMultiReverseGeocodeResponse,
)
from .open_search_multi_reverse_geocode_response_item import (
    OpenSearchMultiReverseGeocodeResponseItem,
)
from .open_search_reverse_geocode_response import OpenSearchReverseGeocodeResponse
from .open_search_search_response import OpenSearchSearchResponse
from .opening_hours import OpeningHours
from .parsing import Parsing
from .phoneme import Phoneme
from .phonemes_section import PhonemesSection
from .physical import Physical
from .point_address_section import PointAddressSection
from .post_autosuggest_mobility_mode import PostAutosuggestMobilityMode
from .post_autosuggest_ranking import PostAutosuggestRanking
from .post_autosuggest_show_item import PostAutosuggestShowItem
from .post_autosuggest_show_map_references_item import (
    PostAutosuggestShowMapReferencesItem,
)
from .post_autosuggest_with_item import PostAutosuggestWithItem
from .post_browse_ranking import PostBrowseRanking
from .post_browse_show_item import PostBrowseShowItem
from .post_discover_mobility_mode import PostDiscoverMobilityMode
from .post_discover_ranking import PostDiscoverRanking
from .post_discover_show_item import PostDiscoverShowItem
from .post_discover_with_item import PostDiscoverWithItem
from .post_multi_revgeocode_show_item import PostMultiRevgeocodeShowItem
from .post_multi_revgeocode_show_map_references_item import (
    PostMultiRevgeocodeShowMapReferencesItem,
)
from .post_multi_revgeocode_show_nav_attributes_item import (
    PostMultiRevgeocodeShowNavAttributesItem,
)
from .post_multi_revgeocode_show_related_item import PostMultiRevgeocodeShowRelatedItem
from .post_multi_revgeocode_types_item import PostMultiRevgeocodeTypesItem
from .post_multi_revgeocode_with_item import PostMultiRevgeocodeWithItem
from .postal_code_details_japan_post import PostalCodeDetailsJapanPost
from .postal_code_details_japan_post_postal_code_type import (
    PostalCodeDetailsJapanPostPostalCodeType,
)
from .postal_code_details_japan_post_postal_entity import (
    PostalCodeDetailsJapanPostPostalEntity,
)
from .postal_code_details_usps_zip import PostalCodeDetailsUspsZip
from .postal_code_details_usps_zip_plus_4 import PostalCodeDetailsUspsZipPlus4
from .postal_code_details_usps_zip_plus_4_postal_code_type import (
    PostalCodeDetailsUspsZipPlus4PostalCodeType,
)
from .postal_code_details_usps_zip_plus_4_postal_entity import (
    PostalCodeDetailsUspsZipPlus4PostalEntity,
)
from .postal_code_details_usps_zip_plus_4_record_type_code import (
    PostalCodeDetailsUspsZipPlus4RecordTypeCode,
)
from .postal_code_details_usps_zip_postal_code_type import (
    PostalCodeDetailsUspsZipPostalCodeType,
)
from .postal_code_details_usps_zip_postal_entity import (
    PostalCodeDetailsUspsZipPostalEntity,
)
from .postal_code_details_usps_zip_zip_classification_code import (
    PostalCodeDetailsUspsZipZipClassificationCode,
)
from .query_term_result_item import QueryTermResultItem
from .range_ import Range
from .rating import Rating
from .rating_media_collection import RatingMediaCollection
from .reference_supplier import ReferenceSupplier
from .reference_supplier_id import ReferenceSupplierId
from .related_address import RelatedAddress
from .related_address_house_number_type import RelatedAddressHouseNumberType
from .related_address_relationship import RelatedAddressRelationship
from .related_address_result_type import RelatedAddressResultType
from .related_result_address import RelatedResultAddress
from .reverse_geocode_result_item import ReverseGeocodeResultItem
from .reverse_geocode_result_item_address_block_type import (
    ReverseGeocodeResultItemAddressBlockType,
)
from .reverse_geocode_result_item_administrative_area_type import (
    ReverseGeocodeResultItemAdministrativeAreaType,
)
from .reverse_geocode_result_item_house_number_type import (
    ReverseGeocodeResultItemHouseNumberType,
)
from .reverse_geocode_result_item_locality_type import (
    ReverseGeocodeResultItemLocalityType,
)
from .reverse_geocode_result_item_result_type import ReverseGeocodeResultItemResultType
from .rgc_address import RgcAddress
from .scoring import Scoring
from .secondary_unit_info import SecondaryUnitInfo
from .segment import Segment
from .speed_limit import SpeedLimit
from .speed_limit_direction import SpeedLimitDirection
from .speed_limit_source import SpeedLimitSource
from .speed_limit_speed_unit import SpeedLimitSpeedUnit
from .street_info import StreetInfo
from .structured_opening_hours import StructuredOpeningHours
from .supplier_reference import SupplierReference
from .time_zone_info import TimeZoneInfo
from .title_and_address_highlighting import TitleAndAddressHighlighting
from .title_highlighting import TitleHighlighting
from .translations_geocode import TranslationsGeocode
from .tripadvisor_image import TripadvisorImage
from .tripadvisor_image_variant import TripadvisorImageVariant
from .tripadvisor_image_variants import TripadvisorImageVariants
from .tripadvisor_media_supplier import TripadvisorMediaSupplier
from .tripadvisor_media_supplier_id import TripadvisorMediaSupplierId
from .truck_amenity_generic import TruckAmenityGeneric
from .truck_amenity_generic_type import TruckAmenityGenericType
from .truck_amenity_showers import TruckAmenityShowers
from .truck_amenity_showers_type import TruckAmenityShowersType
from .truck_fuel import TruckFuel
from .truck_fuel_maximum_truck_class import TruckFuelMaximumTruckClass
from .truck_fuel_type import TruckFuelType

__all__ = (
    "Access",
    "AccessRestrictionAttributes",
    "Address",
    "AddressHighlightingInformation",
    "AddressUsage",
    "AddressUsageUsageType",
    "AdminIdSection",
    "AdminNames",
    "AdminNamesPreference",
    "AutocompleteResultItem",
    "AutocompleteResultItemAdministrativeAreaType",
    "AutocompleteResultItemHouseNumberType",
    "AutocompleteResultItemLocalityType",
    "AutocompleteResultItemResultType",
    "AutosuggestBody",
    "AutosuggestEntityResultItem",
    "AutosuggestEntityResultItemAddressBlockType",
    "AutosuggestEntityResultItemAdministrativeAreaType",
    "AutosuggestEntityResultItemHouseNumberType",
    "AutosuggestEntityResultItemLocalityType",
    "AutosuggestEntityResultItemResultType",
    "AutosuggestQueryResultItem",
    "AutosuggestQueryResultItemResultType",
    "BasicAccessPoint",
    "BrowseBody",
    "BrowseResultItem",
    "BrowseResultItemAddressBlockType",
    "BrowseResultItemAdministrativeAreaType",
    "BrowseResultItemLocalityType",
    "BrowseResultItemResultType",
    "CarFuel",
    "CarFuelType",
    "Category",
    "CategoryRef",
    "Chain",
    "CmVersionSection",
    "Contact",
    "ContactInformation",
    "CountryInfo",
    "DiscoverBody",
    "DisplayResponseCoordinate",
    "Editorial",
    "EditorialMediaCollection",
    "EMobilityServiceProvider",
    "ErrorResponse",
    "EvAvailabilityAttributes",
    "EvAvailabilityConnector",
    "EvAvailabilityEvse",
    "EvAvailabilityEvseState",
    "EvAvailabilityStation",
    "EvChargingAttributes",
    "EvChargingAttributesAccess",
    "EvChargingPoint",
    "EvConnector",
    "EvNameId",
    "EvPaymentSupport",
    "EvPaymentSupportId",
    "EvStation",
    "EvStationConnectorTypeIdsItem",
    "EvStationCurrent",
    "EvStationPaymentMethodIdsItem",
    "ExtendedAccessPoint",
    "ExtendedAccessPointType",
    "ExtendedAttribute",
    "FieldScore",
    "FuelAdditive",
    "FuelAdditiveType",
    "FuelPrice",
    "FuelStation",
    "FuelStationAttributes",
    "FuelStationFuelTypesItem",
    "FuelStationMinimumTruckClass",
    "FunctionalClass",
    "FunctionalClassValue",
    "GeocodeResultItem",
    "GeocodeResultItemAddressBlockType",
    "GeocodeResultItemAdministrativeAreaType",
    "GeocodeResultItemHouseNumberType",
    "GeocodeResultItemLocalityType",
    "GeocodeResultItemResultType",
    "GetAutocompletePostalCodeMode",
    "GetAutocompleteShowItem",
    "GetAutocompleteTypesItem",
    "GetAutosuggestMobilityMode",
    "GetAutosuggestRanking",
    "GetAutosuggestShowItem",
    "GetAutosuggestShowMapReferencesItem",
    "GetAutosuggestWithItem",
    "GetBrowseRanking",
    "GetBrowseShowItem",
    "GetDiscoverMobilityMode",
    "GetDiscoverRanking",
    "GetDiscoverShowItem",
    "GetDiscoverWithItem",
    "GetGeocodeAddressNamesMode",
    "GetGeocodePostalCodeMode",
    "GetGeocodeShowItem",
    "GetGeocodeShowMapReferencesItem",
    "GetGeocodeShowNavAttributesItem",
    "GetGeocodeShowRelatedItem",
    "GetGeocodeShowTranslationsItem",
    "GetGeocodeTypesItem",
    "GetGeocodeWithItem",
    "GetLookupShowItem",
    "GetLookupShowMapReferencesItem",
    "GetLookupShowNavAttributesItem",
    "GetLookupShowRelatedItem",
    "GetRevgeocodeShowItem",
    "GetRevgeocodeShowMapReferencesItem",
    "GetRevgeocodeShowNavAttributesItem",
    "GetRevgeocodeShowRelatedItem",
    "GetRevgeocodeTypesItem",
    "GetRevgeocodeWithItem",
    "ImageMediaCollection",
    "LinkInfoSection",
    "LookupResponse",
    "LookupResponseAddressBlockType",
    "LookupResponseAdministrativeAreaType",
    "LookupResponseClosedPermanently",
    "LookupResponseHouseNumberType",
    "LookupResponseLocalityType",
    "LookupResponseResultType",
    "MapReferenceSection",
    "MapReferenceSectionAS",
    "MapView",
    "MatchInfo",
    "MatchInfoQq",
    "Media",
    "MicroPointAddressSection",
    "MultiResultErrorSection",
    "Name",
    "NameType",
    "NavigationAttributes",
    "OneboxSearchResultItem",
    "OneboxSearchResultItemAddressBlockType",
    "OneboxSearchResultItemAdministrativeAreaType",
    "OneboxSearchResultItemHouseNumberType",
    "OneboxSearchResultItemLocalityType",
    "OneboxSearchResultItemResultType",
    "OpeningHours",
    "OpenSearchAutocompleteResponse",
    "OpenSearchAutosuggestResponse",
    "OpenSearchBrowseResponse",
    "OpenSearchGeocodeResponse",
    "OpenSearchMultiReverseGeocodeErrorResult",
    "OpenSearchMultiReverseGeocodeResponse",
    "OpenSearchMultiReverseGeocodeResponseItem",
    "OpenSearchReverseGeocodeResponse",
    "OpenSearchSearchResponse",
    "Parsing",
    "Phoneme",
    "PhonemesSection",
    "Physical",
    "PointAddressSection",
    "PostalCodeDetailsJapanPost",
    "PostalCodeDetailsJapanPostPostalCodeType",
    "PostalCodeDetailsJapanPostPostalEntity",
    "PostalCodeDetailsUspsZip",
    "PostalCodeDetailsUspsZipPlus4",
    "PostalCodeDetailsUspsZipPlus4PostalCodeType",
    "PostalCodeDetailsUspsZipPlus4PostalEntity",
    "PostalCodeDetailsUspsZipPlus4RecordTypeCode",
    "PostalCodeDetailsUspsZipPostalCodeType",
    "PostalCodeDetailsUspsZipPostalEntity",
    "PostalCodeDetailsUspsZipZipClassificationCode",
    "PostAutosuggestMobilityMode",
    "PostAutosuggestRanking",
    "PostAutosuggestShowItem",
    "PostAutosuggestShowMapReferencesItem",
    "PostAutosuggestWithItem",
    "PostBrowseRanking",
    "PostBrowseShowItem",
    "PostDiscoverMobilityMode",
    "PostDiscoverRanking",
    "PostDiscoverShowItem",
    "PostDiscoverWithItem",
    "PostMultiRevgeocodeShowItem",
    "PostMultiRevgeocodeShowMapReferencesItem",
    "PostMultiRevgeocodeShowNavAttributesItem",
    "PostMultiRevgeocodeShowRelatedItem",
    "PostMultiRevgeocodeTypesItem",
    "PostMultiRevgeocodeWithItem",
    "QueryTermResultItem",
    "Range",
    "Rating",
    "RatingMediaCollection",
    "ReferenceSupplier",
    "ReferenceSupplierId",
    "RelatedAddress",
    "RelatedAddressHouseNumberType",
    "RelatedAddressRelationship",
    "RelatedAddressResultType",
    "RelatedResultAddress",
    "ReverseGeocodeResultItem",
    "ReverseGeocodeResultItemAddressBlockType",
    "ReverseGeocodeResultItemAdministrativeAreaType",
    "ReverseGeocodeResultItemHouseNumberType",
    "ReverseGeocodeResultItemLocalityType",
    "ReverseGeocodeResultItemResultType",
    "RgcAddress",
    "Scoring",
    "SecondaryUnitInfo",
    "Segment",
    "SpeedLimit",
    "SpeedLimitDirection",
    "SpeedLimitSource",
    "SpeedLimitSpeedUnit",
    "StreetInfo",
    "StructuredOpeningHours",
    "SupplierReference",
    "TimeZoneInfo",
    "TitleAndAddressHighlighting",
    "TitleHighlighting",
    "TranslationsGeocode",
    "TripadvisorImage",
    "TripadvisorImageVariant",
    "TripadvisorImageVariants",
    "TripadvisorMediaSupplier",
    "TripadvisorMediaSupplierId",
    "TruckAmenityGeneric",
    "TruckAmenityGenericType",
    "TruckAmenityShowers",
    "TruckAmenityShowersType",
    "TruckFuel",
    "TruckFuelMaximumTruckClass",
    "TruckFuelType",
)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PedestrianTransport")


@_attrs_define
class PedestrianTransport:
    """Information about a transport

    Attributes:
        mode (str): Extensible enum: `pedestrian` `...`
            Pedestrian mode of transport.

            Since the supported pedestrian modes may be extended in the future, the pedestrian mode should be hidden when an
            unknown mode is encountered.
        name (str | Unset): Name of the transport, e.g., the name of a ferry or train.
    """

    mode: str
    name: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        mode = self.mode

        name = self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "mode": mode,
        })
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        mode = d.pop("mode")

        name = d.pop("name", UNSET)

        pedestrian_transport = cls(
            mode=mode,
            name=name,
        )

        pedestrian_transport.additional_properties = d
        return pedestrian_transport

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.place import Place


T = TypeVar("T", bound="Passthrough")


@_attrs_define
class Passthrough:
    """Describes a location and time the section is passing through.

    Attributes:
        place (Place): A place represents a generic location relevant for the route. Example: {'type': 'place',
            'location': {'lat': 50.339167, 'lng': 18.93}}.
        offset (float | Unset): Passthrough offsets are the coordinate index in the polyline.
    """

    place: Place
    offset: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        place = self.place.to_dict()

        offset = self.offset

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "place": place,
        })
        if offset is not UNSET:
            field_dict["offset"] = offset

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.place import Place

        d = dict(src_dict)
        place = Place.from_dict(d.pop("place"))

        offset = d.pop("offset", UNSET)

        passthrough = cls(
            place=place,
            offset=offset,
        )

        passthrough.additional_properties = d
        return passthrough

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.fare_pass_validity_period import FarePassValidityPeriod


T = TypeVar("T", bound="FarePass")


@_attrs_define
class FarePass:
    """Specifies whether this `Fare` is a multi-travel pass, and its characteristics

    Attributes:
        return_journey (bool | Unset): This pass includes the fare for the return journey.
        validity_period (FarePassValidityPeriod | Unset): Specifies a temporal validity period for a pass
        travels (int | Unset): This pass allows for the specified number of travels.
        transfers (int | Unset): Indicates if transfers are permitted with this pass, and if so, how many.
        senior_pass (bool | Unset): This pass is valid only if presented by a senior person.
    """

    return_journey: bool | Unset = UNSET
    validity_period: FarePassValidityPeriod | Unset = UNSET
    travels: int | Unset = UNSET
    transfers: int | Unset = UNSET
    senior_pass: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        return_journey = self.return_journey

        validity_period: dict[str, Any] | Unset = UNSET
        if not isinstance(self.validity_period, Unset):
            validity_period = self.validity_period.to_dict()

        travels = self.travels

        transfers = self.transfers

        senior_pass = self.senior_pass

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if return_journey is not UNSET:
            field_dict["returnJourney"] = return_journey
        if validity_period is not UNSET:
            field_dict["validityPeriod"] = validity_period
        if travels is not UNSET:
            field_dict["travels"] = travels
        if transfers is not UNSET:
            field_dict["transfers"] = transfers
        if senior_pass is not UNSET:
            field_dict["seniorPass"] = senior_pass

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.fare_pass_validity_period import FarePassValidityPeriod

        d = dict(src_dict)
        return_journey = d.pop("returnJourney", UNSET)

        _validity_period = d.pop("validityPeriod", UNSET)
        validity_period: FarePassValidityPeriod | Unset
        if isinstance(_validity_period, Unset):
            validity_period = UNSET
        else:
            validity_period = FarePassValidityPeriod.from_dict(_validity_period)

        travels = d.pop("travels", UNSET)

        transfers = d.pop("transfers", UNSET)

        senior_pass = d.pop("seniorPass", UNSET)

        fare_pass = cls(
            return_journey=return_journey,
            validity_period=validity_period,
            travels=travels,
            transfers=transfers,
            senior_pass=senior_pass,
        )

        fare_pass.additional_properties = d
        return fare_pass

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class NameType(str, Enum):
    ABBREVIATION = "abbreviation"
    AREACODE = "areaCode"
    BASENAME = "baseName"
    EXONYM = "exonym"
    SHORTENED = "shortened"
    SYNONYM = "synonym"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="StructuredOpeningHours")


@_attrs_define
class StructuredOpeningHours:
    """
    Attributes:
        start (str): String with a modified [iCalendar DATE-
            TIME](https://datatracker.ietf.org/doc/html/rfc5545#section-3.3.5) value.
            The date part is omitted, values starts with the time section maker "T". Example: T132000
        duration (str): String with an [iCalendar DURATION](https://datatracker.ietf.org/doc/html/rfc5545#section-3.3.6)
            value.
            A closed day has the value PT00:00M
        recurrence (str): String with a [RECUR](https://datatracker.ietf.org/doc/html/rfc5545#section-3.3.10) rule. Note
            that,
             in contrast to the RFC, the assignment operator is a colon `:` and not an equal sign `=`.
    """

    start: str
    duration: str
    recurrence: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        start = self.start

        duration = self.duration

        recurrence = self.recurrence

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "start": start,
            "duration": duration,
            "recurrence": recurrence,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        start = d.pop("start")

        duration = d.pop("duration")

        recurrence = d.pop("recurrence")

        structured_opening_hours = cls(
            start=start,
            duration=duration,
            recurrence=recurrence,
        )

        structured_opening_hours.additional_properties = d
        return structured_opening_hours

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.notice_severity import NoticeSeverity
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.via_notice_detail import ViaNoticeDetail


T = TypeVar("T", bound="RouteResponseNotice")


@_attrs_define
class RouteResponseNotice:
    """A notice contains important notifications.

    Example:
        {'$ref': '#/components/examples/routeResponseNoticeExample'}

    Attributes:
        code (str): Extensible enum: `noRouteFound` `failedRouteHandleCreation` `cancelled` `routeCalculationFailed`
            `couldNotMatchOrigin` `couldNotMatchDestination` `noReachableChargingStationsFound`
            `violatedTransportModeInRouteHandleDecoding` `unknownError` `routeLengthLimitExceeded` `avoidSegmentsInvalidId`
            `avoidZonesInvalidId` `avoidTruckRoadTypesInvalidId` `returnToRoute` `importFailed` `importSplitRoute`
            `brandDoesNotExist` `unknownParameter` `mainLanguageNotFound` `avoidOptionsWithMakeReachableLimitation`
            `currentWeightChangeNoCurrentWeight` `currentWeightChangeNoGrossWeight` `...`
            Currently known codes (non-exhaustive: this list could be extended for new error situations):

            | Code      | Description  | Severity |
            | --------- | ------- | ------- |
            | noRouteFound | No Route was found | critical |
            | failedRouteHandleCreation | No RouteHandle was created | critical |
            | cancelled | Calculation took too long and was cancelled | critical |
            | routeCalculationFailed | Calculation did not succeed | critical |
            | couldNotMatchOrigin | Origin waypoint could not be matched | critical |
            | couldNotMatchDestination | Destination waypoint could not be matched | critical |
            | noReachableChargingStationsFound | Initial charge is not enough to reach any known charging stations |
            critical |
            | violatedTransportModeInRouteHandleDecoding | Route handle decoding failed due to forbidden segments for the
            specified transport mode | critical |
            | unknownError | No detailed error cause has been determined | critical |
            | routeLengthLimitExceeded | Distance between waypoints is too large for current options | critical |
            | avoidSegmentsInvalidId | The provided segment ID was not found | info |
            | avoidZonesInvalidId | The provided zone ID was not found | info |
            | avoidTruckRoadTypesInvalidId | The provided truck road type id was not found | info |
            | returnToRoute | Applicable only to requests with route handle provided. Current route position was not on the
            original route. New route was calculated from the current position to the destination. Old route may have been
            reused. | info |
            | importFailed | No route section was found for imported waypoints | critical |
            | importSplitRoute | Not all trace points were matched | info |
            | brandDoesNotExist | Preferred charging station brand does not exist | info |
            | unknownParameter | The provided parameter is unknown | info |
            | mainLanguageNotFound | The first language in the list of preferred languages is not supported. | info |
            | avoidOptionsWithMakeReachableLimitation | Avoiding toll or controlledAccessHighway with makeReachable is
            supported only for trucks up to 7.5 tons. | critical |
            | currentWeightChangeNoCurrentWeight | Route handle contains `currentWeightChange` but missing
            vehicle[currentWeight]. `currentWeight` will not be evaluated. | info |
            | currentWeightChangeNoGrossWeight | Route handle contains `currentWeightChange` but missing
            vehicle[grossWeight]. Vehicle restrictions based on gross weight will not be evaluated. | info |
             Example: noRouteFound.
        title (str | Unset): Human-readable notice description. Example: No route found.
        severity (NoticeSeverity | Unset): Describes the impact a notice has on the resource to which the notice is
            attached.
            * critical - The notice must not be ignored, even if the type of notice is not known to the user. Any associated
            resource (e.g., route section) must not be used without further evaluation.
            * info - The notice is for informative purposes, but does not affect usability of the route.
        details (list[ViaNoticeDetail] | Unset): Additional details about the notice
    """

    code: str
    title: str | Unset = UNSET
    severity: NoticeSeverity | Unset = UNSET
    details: list[ViaNoticeDetail] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        code = self.code

        title = self.title

        severity: str | Unset = UNSET
        if not isinstance(self.severity, Unset):
            severity = self.severity.value

        details: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.details, Unset):
            details = []
            for details_item_data in self.details:
                details_item = details_item_data.to_dict()
                details.append(details_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "code": code,
        })
        if title is not UNSET:
            field_dict["title"] = title
        if severity is not UNSET:
            field_dict["severity"] = severity
        if details is not UNSET:
            field_dict["details"] = details

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.via_notice_detail import ViaNoticeDetail

        d = dict(src_dict)
        code = d.pop("code")

        title = d.pop("title", UNSET)

        _severity = d.pop("severity", UNSET)
        severity: NoticeSeverity | Unset
        if isinstance(_severity, Unset):
            severity = UNSET
        else:
            severity = NoticeSeverity(_severity)

        _details = d.pop("details", UNSET)
        details: list[ViaNoticeDetail] | Unset = UNSET
        if _details is not UNSET:
            details = []
            for details_item_data in _details:
                details_item = ViaNoticeDetail.from_dict(details_item_data)

                details.append(details_item)

        route_response_notice = cls(
            code=code,
            title=title,
            severity=severity,
            details=details,
        )

        route_response_notice.additional_properties = d
        return route_response_notice

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PostMultiRevgeocodeShowNavAttributesItem(str, Enum):
    ACCESS = "access"
    FUNCTIONALCLASS = "functionalClass"
    PHYSICAL = "physical"
    SPEEDLIMITS = "speedLimits"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.fuel_additive_type import FuelAdditiveType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.fuel_price import FuelPrice


T = TypeVar("T", bound="FuelAdditive")


@_attrs_define
class FuelAdditive:
    """
    Attributes:
        type_ (FuelAdditiveType): **BETA, RESTRICTED**

            Indicates the type or brand of Diesel Emission Fluid (DEF) available.
        available (bool): **BETA, RESTRICTED**

            When set to:
            - `true` indicates that Diesel Emission Fluid (DEF) is available at this place
            - `false` indicates that Diesel Emission Fluid (DEF) is not available at this place
        in_cans (bool | Unset): **BETA, RESTRICTED**

            When set to:
            - `true` indicates that Diesel Emission Fluid (DEF) is available in cans.
            - `false` indicates that Diesel Emission Fluid (DEF) is not available in cans.
        at_the_pump (bool | Unset): **BETA, RESTRICTED**

            When set to:
            - `true` indicates that Diesel Emission Fluid (DEF) is available at the pump.
            - `false` indicates that Diesel Emission Fluid (DEF) is not available at the pump.
        price (FuelPrice | Unset):
    """

    type_: FuelAdditiveType
    available: bool
    in_cans: bool | Unset = UNSET
    at_the_pump: bool | Unset = UNSET
    price: FuelPrice | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        available = self.available

        in_cans = self.in_cans

        at_the_pump = self.at_the_pump

        price: dict[str, Any] | Unset = UNSET
        if not isinstance(self.price, Unset):
            price = self.price.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "available": available,
        })
        if in_cans is not UNSET:
            field_dict["inCans"] = in_cans
        if at_the_pump is not UNSET:
            field_dict["atThePump"] = at_the_pump
        if price is not UNSET:
            field_dict["price"] = price

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.fuel_price import FuelPrice

        d = dict(src_dict)
        type_ = FuelAdditiveType(d.pop("type"))

        available = d.pop("available")

        in_cans = d.pop("inCans", UNSET)

        at_the_pump = d.pop("atThePump", UNSET)

        _price = d.pop("price", UNSET)
        price: FuelPrice | Unset
        if isinstance(_price, Unset):
            price = UNSET
        else:
            price = FuelPrice.from_dict(_price)

        fuel_additive = cls(
            type_=type_,
            available=available,
            in_cans=in_cans,
            at_the_pump=at_the_pump,
            price=price,
        )

        fuel_additive.additional_properties = d
        return fuel_additive

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Sequence
from dataclasses import KW_ONLY, dataclass
from datetime import datetime
from typing import Any

from geopandas import GeoDataFrame
from shapely import (
    MultiLineString,
    MultiPolygon,
    Point,
)

from . import _consts, _util
from ._client import CasaGeoClient
from ._types import CasaGeoResult


def _position_point(data: dict | None) -> Point | None:
    if data is None:
        return None
    try:
        if (elv := data.get("elv")) is not None:
            return Point(data["lng"], data["lat"], elv)
        return Point(data["lng"], data["lat"])
    except KeyError:
        # FIXME: Raise warning here.
        return None


def _fromisoformat(date: str | None) -> datetime | None:
    if date is None:
        return None
    try:
        return datetime.fromisoformat(date)
    except ValueError:
        # FIXME: Raise warning here.
        return None


class IsolinesResult(CasaGeoResult):
    """
    Represents the result of a CasaGeoSpatial.isolines() query.
    """

    @staticmethod
    def _geometry(isoline: dict) -> MultiPolygon | None:
        try:
            polygons = isoline["polygons"]
        except KeyError:
            return None

        return MultiPolygon([
            (
                _util.flexpolyline_points(outer),
                [
                    _util.flexpolyline_points(inner)
                    for inner in polygon.get("inner", ())
                ],
            )
            for polygon in polygons
            if (outer := polygon.get("outer")) is not None
        ])

    @staticmethod
    def _range_type(isoline: dict) -> str | None:
        try:
            return isoline["range"]["type"]
        except KeyError:
            return None

    @staticmethod
    def _range_value(isoline: dict) -> float | None:
        try:
            value = float(isoline["range"]["value"])
        except KeyError:
            return None

        # We interpret range_values as minutes, while HERE interprets them as seconds.
        if IsolinesResult._range_type(isoline) == "time":
            value /= 60

        return value

    def dataframe(
        self,
        id_: Any | None = None,
        *,
        departure_info: bool = False,
        arrival_info: bool = False,
    ) -> GeoDataFrame:
        """
        Return the result as a GeoDataFrame.

        The dataframe will contain the following columns:

        - *id* (str | Any): Fixed identifier added to each row. Can be
          overridden using the ``id_`` parameter.
        - *subid* (int): Numeric index of the result value starting from zero.
        - *geometry* (MultiPolygon): A MultiPolygon representing the isoline’s geometry.
        - *rangetype* (str): String representing the type of the distance value.
        - *rangevalue* (float): The distance value represented by this range.
        - *timestamp* (datetime): Timestamp of when this result was created.

        When ``departure_info`` is ``True``, the dataframe also contains the
        columns below. If the direction of the query was not ``"outgoing"``, all
        these values will be null.

        - *departure_time* (datetime): Timestamp representing the expected
          departure time.
        - *departure_placename* (str): Name of the departure location.
        - *departure_position* (Point): Resolved position of the departure
          location used for route calculation.
        - *departure_displayposition* (Point): Position of a map marker
          referring to the departure location.
        - *departure_queryposition* (Point): The original position provided
          in the request.

        When ``arrival_info`` is ``True``, the dataframe also contains the
        columns below. If the direction of the query was not ``"incoming"``, all
        these values will be null.

        - *arrival_time* (datetime): Timestamp representing the expected
          arrival time.
        - *arrival_placename* (str): Name of the arrival location.
        - *arrival_position* (Point): Resolved position of the arrival
          location used for route calculation.
        - *arrival_displayposition* (Point): Position of a map marker
          referring to the arrival location.
        - *arrival_queryposition* (Point): The original position provided
          in the request.

        Args:
            id_: Fixed identifier to be added to each row.
            departure_info: Include additional info about the departure
                time and location.
            arrival_info: Include additional info about the arrival
                time and location.

        Returns:
            GeoDataFrame: The list of computed isolines as a GeoDataFrame.
        """
        # - *names*: String descriptor of an isoline, generated from 'range_type' and 'range_value'.

        if id_ is None:
            id_ = str(self._uuid)

        data: list[dict] = []
        for index, isoline in enumerate(self._data.get("isolines", [])):
            data.append(row := {})

            if True:
                # fmt: off
                row["id"]               = id_
                row["subid"]            = index
                row["geometry"]         = self._geometry(isoline)
                row["rangetype"]        = self._range_type(isoline)
                row["rangevalue"]       = self._range_value(isoline)
                row["timestamp"]        = self._timestamp
                # fmt: on

        if not data:
            return GeoDataFrame()

        # TODO: When elevation is returned, the CRS should be different (EPSG:4979 ?).
        df = GeoDataFrame(data, geometry="geometry", crs="EPSG:4326")

        if departure_info:
            departure = self.departure_info() or {}
            # fmt: off
            df["departure_time"]            = departure.get("time")
            df["departure_placename"]       = departure.get("placename")
            df["departure_position"]        = departure.get("position")
            df["departure_displayposition"] = departure.get("displayposition")
            df["departure_queryposition"]   = departure.get("queryposition")
            # fmt: on

        if arrival_info:
            arrival = self.arrival_info() or {}
            # fmt: off
            df["arrival_time"]              = arrival.get("time")
            df["arrival_placename"]         = arrival.get("placename")
            df["arrival_position"]          = arrival.get("position")
            df["arrival_displayposition"]   = arrival.get("displayposition")
            df["arrival_queryposition"]     = arrival.get("queryposition")
            # fmt: on

        return df

    def has_departure_info(self) -> bool:
        """Return ``True`` if the API response includes departure information."""
        return "departure" in self._data

    def departure_info(self) -> dict | None:
        """
        Return departure information, if available.

        Departure information includes the following values, all of which can be
        ``None``:

        - *time* (datetime): Timestamp representing the expected departure time.
        - *placename* (str): Name of the departure location.
        - *position* (Point): Resolved position of the departure location used
          for route calculation.
        - *displayposition* (Point): Position of a map marker referring to the
          departure location.
        - *queryposition* (Point): The original departure position specified in
          the request.

        Returns:
            dict | None: Departure information as a ``dict``, or ``None`` if no
            departure information is available.
        """
        try:
            departure = self._data["departure"]
            place = departure["place"]
        except KeyError:
            return None
        return {
            "time": _fromisoformat(departure.get("time")),
            "placename": place.get("name"),
            "position": _position_point(place.get("location")),
            "displayposition": _position_point(place.get("displayLocation")),
            "queryposition": _position_point(place.get("originalLocation")),
        }

    def has_arrival_info(self) -> bool:
        """Return ``True`` if the API response includes arrival information."""
        return "arrival" in self._data

    def arrival_info(self) -> dict | None:
        """
        Return arrival information, if available.

        Arrival information includes the following values, all of which can be
        ``None``:

        - *time* (datetime): Timestamp representing the expected arrival time.
        - *placename* (str): Name of the arrival location.
        - *position* (Point): Resolved position of the arrival location used
          for route calculation.
        - *displayposition* (Point): Position of a map marker referring to the
          arrival location.
        - *queryposition* (Point): The original arrival position specified in
          the request.

        Returns:
            dict | None: Arrival information as a ``dict``, or ``None`` if no
            arrival information is available.
        """
        try:
            arrival = self._data["arrival"]
            place = arrival["place"]
        except KeyError:
            return None
        return {
            "time": _fromisoformat(arrival.get("time")),
            "placename": place.get("name"),
            "position": _position_point(place.get("location")),
            "displayposition": _position_point(place.get("displayLocation")),
            "queryposition": _position_point(place.get("originalLocation")),
        }


class RoutesResult(CasaGeoResult):
    """
    Represents the result of a CasaGeoSpatial.routes() query.
    """

    @staticmethod
    def _geometry(route: dict) -> MultiLineString | None:
        try:
            return MultiLineString([
                _util.flexpolyline_points(section["polyline"])
                for section in route["sections"]
            ])
        except KeyError:
            return None

    def dataframe(
        self,
        id_: Any | None = None,
        *,
        departure_info: bool = False,
        arrival_info: bool = False,
    ) -> GeoDataFrame:
        """
        Return the result as a GeoDataFrame.

        The dataframe will contain the following columns:

        - *id* (str | Any): Fixed identifier added to each row. Can be
          overridden using the ``id_`` parameter.
        - *subid* (int): Numeric index of the result value starting from zero.
        - *routeid* (str): Unique identifier of the route.
        - *geometry* (MultiLineString): Geometry of the route by sections.
        - *timestamp* (datetime): Timestamp of when this result was created.

        When ``departure_info`` is ``True``, the dataframe also contains the
        following columns:

        - *departure_time* (datetime): Timestamp representing the expected
          departure time.
        - *departure_placename* (str): Name of the departure location.
        - *departure_position* (Point): Resolved position of the departure
          location used for route calculation.
        - *departure_displayposition* (Point): Position of a map marker
          referring to the departure location.
        - *departure_queryposition* (Point): The original departure position
          specified in the request.

        When ``arrival_info`` is ``True``, the dataframe also contains the
        following columns:

        - *arrival_time* (datetime): Timestamp representing the expected arrival
          time.
        - *arrival_placename* (str): Name of the arrival location.
        - *arrival_position* (Point): Resolved position of the arrival location
          used for route calculation.
        - *arrival_displayposition* (Point): Position of a map marker referring
          to the arrival location.
        - *arrival_queryposition* (Point): The original arrival position
          specified in the request.

        Args:
            id_: Fixed identifier to be added to each row.
            departure_info: Include additional info about the departure
                time and location of each route.
            arrival_info: Include additional info about the arrival
                time and location of each route.

        Returns:
            GeoDataFrame: The list of computed routes as a GeoDataFrame.
        """

        if id_ is None:
            id_ = str(self._uuid)

        data: list[dict] = []
        for index, route in enumerate(self._data.get("routes", [])):
            data.append(row := {})

            if True:
                # fmt: off
                row["id"]                           = id_
                row["subid"]                        = index
                row["routeid"]                      = route.get("id")
                row["geometry"]                     = self._geometry(route)
                row["timestamp"]                    = self._timestamp
                # fmt: on

            if departure_info:
                departure = self.departure_info(index) or {}
                # fmt: off
                row["departure_time"]               = departure.get("time")
                row["departure_placename"]          = departure.get("placename")
                row["departure_position"]           = departure.get("position")
                row["departure_displayposition"]    = departure.get("displayposition")
                row["departure_queryposition"]      = departure.get("queryposition")
                # fmt: on

            if arrival_info:
                arrival = self.arrival_info(index) or {}
                # fmt: off
                row["arrival_time"]                 = arrival.get("time")
                row["arrival_placename"]            = arrival.get("placename")
                row["arrival_position"]             = arrival.get("position")
                row["arrival_displayposition"]      = arrival.get("displayposition")
                row["arrival_queryposition"]        = arrival.get("queryposition")
                # fmt: on

        if not data:
            return GeoDataFrame()

        # TODO: When elevation is returned, the CRS should be different (EPSG:4979 ?).
        return GeoDataFrame(data, geometry="geometry", crs="EPSG:4326")

    def departure_info(self, route: int, section: int = 0) -> dict | None:
        """
        Returns departure information for a specific route or section.

        Departure information includes the following values, all of which can be
        ``None``:

        - *time* (datetime): Timestamp representing the expected departure time.
        - *placename* (str): Name of the departure location.
        - *position* (Point): Resolved position of the departure location used
          for route calculation.
        - *displayposition* (Point): Position of a map marker referring to the
          departure location.
        - *queryposition* (Point): The original departure position specified in
          the request.

        Args:
            route: Index of the route in the result list.
            section: Index of the section within the route, defaults to the
                first section.

        Returns:
            dict | None: Departure information as a ``dict``, or ``None`` if no
            such information exists for the specified route and section.
        """
        try:
            departure = self._data["routes"][route]["sections"][section]["departure"]
            place = departure["place"]
        except (KeyError, IndexError):
            return None
        return {
            "time": _fromisoformat(departure.get("time")),
            "placename": place.get("name"),
            "position": _position_point(place.get("location")),
            "displayposition": _position_point(place.get("displayLocation")),
            "queryposition": _position_point(place.get("originalLocation")),
        }

    def arrival_info(self, route: int, section: int = -1) -> dict | None:
        """
        Returns arrival information for a specific route or section.

        Arrival information includes the following values, all of which can be
        ``None``:

        - *time* (datetime): Timestamp representing the expected arrival time.
        - *placename* (str): Name of the arrival location.
        - *position* (Point): Resolved position of the arrival location used
          for route calculation.
        - *displayposition* (Point): Position of a map marker referring to the
          arrival location.
        - *queryposition* (Point): The original arrival position specified in
          the request.

        Args:
            route: Index of the route in the result list.
            section: Index of the section within the route, defaults to the
                last section.

        Returns:
            dict | None: Arrival information as a ``dict``, or ``None`` if no
            such information exists for the specified route and section.
        """
        try:
            arrival = self._data["routes"][route]["sections"][section]["arrival"]
            place = arrival["place"]
        except (KeyError, IndexError):
            return None
        return {
            "time": _fromisoformat(arrival.get("time")),
            "placename": place.get("name"),
            "position": _position_point(place.get("location")),
            "displayposition": _position_point(place.get("displayLocation")),
            "queryposition": _position_point(place.get("originalLocation")),
        }


@dataclass
class CasaGeoSpatial:
    """
    Provides spatial calculations such as routing and isolines.

    Attributes:
        client:
            The client object authorizing these queries.

        language:
            The preferred language for the response.

            This must be a valid IETF BCP47 language tag, such as "en-US", or a
            comma-separated list of such tags in order of preference.

        unit_system:
            The system of units to use for localized quantities, either
            ``"metric"`` or ``"imperial"``.

        transport_mode:
            The mode of transport to use for routing, e.g. ``"car"``.

            The following modes are available:

            - ``"car"``
            - ``"pedestrian"``
            - ``"bicycle"``

        routing_mode:
            Whether to prefer ``"fast"`` or ``"short"`` routes.

        direction:
            The direction of travel for isolines, either ``"outgoing"`` or
            ``"incoming"``.

        departure_time:
            The date and time of departure for time-dependent routing. This
            value is only used when ``direction`` is ``"outgoing"``.

        arrival_time:
            The date and time of arrival for time-dependent routing. This
            value is only used when ``direction`` is ``"incoming"``.

        consider_traffic:
            Whether to consider traffic data during routing.

            When using this feature, you should also specify either
            ``departure_time`` or ``arrival_time``, depending on ``direction``,
            to get meaningful and reproducible results.

        avoid_features:
            Route features to avoid during routing, e.g. ``("ferry", "tollRoad")``.

            The following features are currently supported:

            - ``"carShuttleTrain"``
            - ``"controlledAccessHighway"``
            - ``"dirtRoad"``
            - ``"ferry"``
            - ``"seasonalClosure"``
            - ``"tollRoad"``
            - ``"tunnel"``

        exclude_countries:
            Countries to exclude from routing, e.g. ``("USA", "DEU")``.

            Countries must be specified by their ISO 3166-1 alpha-3 country
            codes.
    """

    client: CasaGeoClient
    _: KW_ONLY
    language: str | None = None
    unit_system: str | None = None
    transport_mode: str = "car"
    routing_mode: str = "fast"
    direction: str = "outgoing"
    departure_time: datetime | None = None
    arrival_time: datetime | None = None
    consider_traffic: bool = False
    avoid_features: Sequence[str] = ()
    exclude_countries: Sequence[str] = ()

    def __post_init__(self) -> None:
        # Check that self.client is not a string, because the beta version of
        # this class took the license key as the first parameter.
        if not isinstance(self.client, CasaGeoClient):
            raise TypeError("client must be a CasaGeoClient instance")
        if self.language is None:
            self.language = self.client.preferred_language
        if self.unit_system is None:
            self.unit_system = self.client.preferred_unit_system

    def _validate(self) -> None:
        if self.language is not None:
            if not self.language:
                raise ValueError("language must not be empty")
            for tag in self.language.split(","):
                _util.validate_ietf_bcp47_language_tag(tag)

        if self.unit_system is not None:
            if self.unit_system not in _consts.UNIT_SYSTEMS:
                raise ValueError(
                    f"unit_system must be one of {_consts.UNIT_SYSTEMS}, got {self.unit_system!r}"
                )

        if self.transport_mode not in _consts.TRANSPORT_MODES:
            raise ValueError(
                f"transport_mode must be one of {_consts.TRANSPORT_MODES}, got {self.transport_mode!r}"
            )

        if self.routing_mode not in _consts.ROUTING_MODES:
            raise ValueError(
                f"routing_mode must be one of {_consts.ROUTING_MODES}, got {self.routing_mode!r}"
            )

        if self.direction not in _consts.DIRECTION_TYPES:
            raise ValueError(
                f"direction must be one of {_consts.DIRECTION_TYPES}, got {self.direction!r}"
            )

        for feature in self.avoid_features:
            if feature not in _consts.AVOIDABLE_FEATURES:
                raise ValueError(
                    f"avoid_features may only contain {_consts.AVOIDABLE_FEATURES}, got {feature!r}"
                )

        for country in self.exclude_countries:
            _util.validate_iso3166_alpha3_country_code(country)

    def _departure(self) -> str | None:
        if self.direction != "outgoing":
            return None
        if self.departure_time is None:
            return "any"
        return self.departure_time.isoformat()

    def _arrival(self) -> str | None:
        if self.direction != "incoming":
            return None
        if self.arrival_time is None:
            return "any"
        return self.arrival_time.isoformat()

    # https://www.here.com/docs/bundle/isoline-routing-api-developer-guide-v8/page/README.html
    # https://www.here.com/docs/bundle/isoline-routing-api-v8-api-reference/page/index.html
    @IsolinesResult.wrap_errors()
    def isolines(
        self,
        position: Point,
        ranges: Sequence[float],
        *,
        range_type: str = "time",
    ) -> IsolinesResult:
        """
        Calculate isolines around ``position``.

        Args:
            position:
                The center point of the isolines.
            ranges:
                The distances (in meters) or durations (in minutes) for which to
                calculate isolines.
            range_type:
                The type of range to calculate. Must be one of ``"time"`` or
                ``"distance"``.

        Returns:
            IsolinesResult: The result of the isoline calculation.
        """
        from ._apiclient.isolines.api.isoline import calculate_isolines
        from ._apiclient.isolines.models import (
            Avoid,
            Exclude,
            Range,
            RouterMode,
            RoutingMode,
            Traffic,
            TrafficMode,
        )
        from ._apiclient.isolines.types import UNSET

        self._validate()

        if not position.is_valid:
            raise ValueError("position must be a valid 2D point")

        if range_type not in _consts.RANGE_TYPES:
            raise ValueError(
                f"range_type must be one of {_consts.RANGE_TYPES}, got {range_type!r}"
            )

        # We interpret time values as minutes, while HERE interprets them as seconds.
        if range_type == "time":
            ranges = [x * 60 for x in ranges]

        point_str = _util.point_to_latlng(position)
        departure = self.departure_time or UNSET
        arrival = self.arrival_time or UNSET

        response = calculate_isolines.sync_detailed(
            client=self.client._isolines_client,
            range_=Range(
                type_=range_type,
                values=",".join(str(int(x)) for x in ranges),
            ),
            origin=(point_str if self.direction == "outgoing" else UNSET),
            destination=(point_str if self.direction == "incoming" else UNSET),
            departure_time=(departure if self.direction == "outgoing" else UNSET),
            arrival_time=(arrival if self.direction == "incoming" else UNSET),
            transport_mode=RouterMode(self.transport_mode),
            routing_mode=RoutingMode(self.routing_mode),
            avoid=Avoid(
                features=",".join(self.avoid_features)
                if self.avoid_features
                else UNSET,
            ),
            exclude=Exclude(
                countries=",".join(self.exclude_countries)
                if self.exclude_countries
                else UNSET
            ),
            traffic=Traffic(
                mode=TrafficMode.DEFAULT
                if self.consider_traffic
                else TrafficMode.DISABLED
            ),
        )

        return IsolinesResult.from_response(response)

    # https://www.here.com/docs/bundle/routing-api-developer-guide-v8/page/README.html
    # https://www.here.com/docs/bundle/routing-api-v8-api-reference/page/index.html
    @RoutesResult.wrap_errors()
    def routes(
        self,
        origin: Point,
        destination: Point,
        *,
        alternatives: int = 0,
    ) -> RoutesResult:
        """
        Calculate routes from ``origin`` to ``destination``.

        Args:
            origin: The starting point of the route.
            destination: The destination point of the route.
            alternatives: The number of alternate routes to calculate.

        Returns:
            RoutesResult: The result of the route calculation.
        """
        from ._apiclient.router.api.routing import calculate_routes
        from ._apiclient.router.models import (
            Avoid,
            Exclude,
            Return,
            RouterMode,
            RoutingMode,
            Traffic,
            TrafficMode,
            Units,
        )
        from ._apiclient.router.types import UNSET

        self._validate()

        if not origin.is_valid:
            raise ValueError("origin must be a valid 2D point")

        if not destination.is_valid:
            raise ValueError("destination must be a valid 2D point")

        if not 0 <= alternatives <= 6:
            raise ValueError(
                "Number of alternate routes must be between 0 and 6 inclusive"
            )

        # NOTE: Due to API client limitations, we can’t support departureTime=any and
        #       arrivalTime=any everywhere, so we choose to not support it at all.
        departure = self.departure_time.isoformat() if self.departure_time else UNSET
        arrival = self.arrival_time or UNSET

        response = calculate_routes.sync_detailed(
            client=self.client._routing_client,
            origin=_util.point_to_latlng(origin),
            destination=_util.point_to_latlng(destination),
            alternatives=alternatives,
            departure_time=(departure if self.direction == "outgoing" else UNSET),
            arrival_time=(arrival if self.direction == "incoming" else UNSET),
            transport_mode=RouterMode(self.transport_mode),
            routing_mode=RoutingMode(self.routing_mode),
            avoid=Avoid(
                features=",".join(self.avoid_features)
                if self.avoid_features
                else UNSET,
            ),
            exclude=Exclude(
                countries=",".join(self.exclude_countries)
                if self.exclude_countries
                else UNSET
            ),
            traffic=Traffic(
                mode=TrafficMode.DEFAULT
                if self.consider_traffic
                else TrafficMode.DISABLED
            ),
            lang=self.language.split(",") if self.language is not None else UNSET,
            units=Units(self.unit_system) if self.unit_system is not None else UNSET,
            return_=[Return.SUMMARY, Return.POLYLINE],
        )

        return RoutesResult.from_response(response)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ChargingConnectorAttributes")


@_attrs_define
class ChargingConnectorAttributes:
    """Details of the connector that is suggested to be used in the section's `postAction` for charging.

    Attributes:
        power (float): Power supplied by the suggested connector in kW.
        supply_type (str): Extensible enum: `acSingle` `acThree` `dc` `...`
            Currently possible values are:

            * `acSingle` : Single phase Alternating Current supply
            * `acThree`: Three phase Alternating Current supply
            * `dc`: Direct Current supply
        connector_type (str): Extensible enum: `iec62196Type1Combo` `iec62196Type2Combo` `chademo` `tesla` `saeJ3400`
            `gbtDc` `unknown` `...`
            Currently supported connector types are:
            * `iec62196Type1Combo`:  Type 1 Combo connector, commonly called "CCS1"
            * `iec62196Type2Combo`:  Type 2 Combo connector, commonly called "CCS2"
            * `chademo`: CHAdeMO connector
            * `tesla`: North American Charging Standard (NACS) connector. This connector type is deprecated. Return value
            used instead of `saeJ3400` when the user specifies the deprecated value `tesla` in `ev[connectorTypes]`
            * `saeJ3400`: North American Charging System (NACS) connector
            * `gbtDc`: GB/T Guobiao GB/T 20234.3 DC connector
            * `unknown`: Connector type is not known, e.g., if station is provided by user in request
        current (float | Unset): Current of the suggested connector in Amperes.
        voltage (float | Unset): Voltage of the suggested connector in Volts.
    """

    power: float
    supply_type: str
    connector_type: str
    current: float | Unset = UNSET
    voltage: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        power = self.power

        supply_type = self.supply_type

        connector_type = self.connector_type

        current = self.current

        voltage = self.voltage

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "power": power,
            "supplyType": supply_type,
            "connectorType": connector_type,
        })
        if current is not UNSET:
            field_dict["current"] = current
        if voltage is not UNSET:
            field_dict["voltage"] = voltage

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        power = d.pop("power")

        supply_type = d.pop("supplyType")

        connector_type = d.pop("connectorType")

        current = d.pop("current", UNSET)

        voltage = d.pop("voltage", UNSET)

        charging_connector_attributes = cls(
            power=power,
            supply_type=supply_type,
            connector_type=connector_type,
            current=current,
            voltage=voltage,
        )

        charging_connector_attributes.additional_properties = d
        return charging_connector_attributes

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="SinglePrice")


@_attrs_define
class SinglePrice:
    """
    Attributes:
        type_ (str): Type of price represented by this object. The API customer is responsible for correctly visualizing
            the pricing model. As it is possible to extend the supported price types in the future,
            the price information should be hidden when an unknown type is encountered.

            Available price types are:

              * `value` - A single value.
              * `range` - A range value that includes a minimum and maximum price.
        currency (str): Local currency of the price compliant to ISO 4217
        value (float): The price value
        estimated (bool | Unset): Attribute value is `true` if the fare price is estimated, `false` if it is an exact
            value. Default: False.
        unit (int | Unset): Duration in seconds. Example: 198.
    """

    type_: str
    currency: str
    value: float
    estimated: bool | Unset = False
    unit: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        currency = self.currency

        value = self.value

        estimated = self.estimated

        unit = self.unit

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "currency": currency,
            "value": value,
        })
        if estimated is not UNSET:
            field_dict["estimated"] = estimated
        if unit is not UNSET:
            field_dict["unit"] = unit

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = d.pop("type")

        currency = d.pop("currency")

        value = d.pop("value")

        estimated = d.pop("estimated", UNSET)

        unit = d.pop("unit", UNSET)

        single_price = cls(
            type_=type_,
            currency=currency,
            value=value,
            estimated=estimated,
            unit=unit,
        )

        single_price.additional_properties = d
        return single_price

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class V1IsolinesCreateRangetype(str, Enum):
    CONSUMPTION = "consumption"
    DISTANCE = "distance"
    TIME = "time"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="TransitTransport")


@_attrs_define
class TransitTransport:
    """Transit transport information.

    Attributes:
        mode (str): Extensible enum: `highSpeedTrain` `intercityTrain` `interRegionalTrain` `regionalTrain` `cityTrain`
            `bus` `ferry` `subway` `lightRail` `privateBus` `inclined` `aerial` `busRapid` `monorail` `carShuttleTrain`
            `flight` `spaceship` `...`
            Transit mode of transport in the route response
        name (str | Unset): Transit line name Example: U2.
        headsign (str | Unset): Transit line headsign
        category (str | Unset): Human readable transport category (such as Bus, Gondola, Tram, Train, ...) Example:
            Train.
        color (str | Unset):  Example: #FF0000.
        text_color (str | Unset):  Example: #FF0000.
        short_name (str | Unset): Short name of a transit line. Example: U2.
        long_name (str | Unset): Long name of a transit line. Example: Pankow - Ruhleben.
    """

    mode: str
    name: str | Unset = UNSET
    headsign: str | Unset = UNSET
    category: str | Unset = UNSET
    color: str | Unset = UNSET
    text_color: str | Unset = UNSET
    short_name: str | Unset = UNSET
    long_name: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        mode = self.mode

        name = self.name

        headsign = self.headsign

        category = self.category

        color = self.color

        text_color = self.text_color

        short_name = self.short_name

        long_name = self.long_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "mode": mode,
        })
        if name is not UNSET:
            field_dict["name"] = name
        if headsign is not UNSET:
            field_dict["headsign"] = headsign
        if category is not UNSET:
            field_dict["category"] = category
        if color is not UNSET:
            field_dict["color"] = color
        if text_color is not UNSET:
            field_dict["textColor"] = text_color
        if short_name is not UNSET:
            field_dict["shortName"] = short_name
        if long_name is not UNSET:
            field_dict["longName"] = long_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        mode = d.pop("mode")

        name = d.pop("name", UNSET)

        headsign = d.pop("headsign", UNSET)

        category = d.pop("category", UNSET)

        color = d.pop("color", UNSET)

        text_color = d.pop("textColor", UNSET)

        short_name = d.pop("shortName", UNSET)

        long_name = d.pop("longName", UNSET)

        transit_transport = cls(
            mode=mode,
            name=name,
            headsign=headsign,
            category=category,
            color=color,
            text_color=text_color,
            short_name=short_name,
            long_name=long_name,
        )

        transit_transport.additional_properties = d
        return transit_transport

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.editorial_media_collection import EditorialMediaCollection
    from ..models.image_media_collection import ImageMediaCollection
    from ..models.rating_media_collection import RatingMediaCollection


T = TypeVar("T", bound="Media")


@_attrs_define
class Media:
    """
    Attributes:
        images (ImageMediaCollection | Unset):
        editorials (EditorialMediaCollection | Unset):
        ratings (RatingMediaCollection | Unset):
    """

    images: ImageMediaCollection | Unset = UNSET
    editorials: EditorialMediaCollection | Unset = UNSET
    ratings: RatingMediaCollection | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        images: dict[str, Any] | Unset = UNSET
        if not isinstance(self.images, Unset):
            images = self.images.to_dict()

        editorials: dict[str, Any] | Unset = UNSET
        if not isinstance(self.editorials, Unset):
            editorials = self.editorials.to_dict()

        ratings: dict[str, Any] | Unset = UNSET
        if not isinstance(self.ratings, Unset):
            ratings = self.ratings.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if images is not UNSET:
            field_dict["images"] = images
        if editorials is not UNSET:
            field_dict["editorials"] = editorials
        if ratings is not UNSET:
            field_dict["ratings"] = ratings

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.editorial_media_collection import EditorialMediaCollection
        from ..models.image_media_collection import ImageMediaCollection
        from ..models.rating_media_collection import RatingMediaCollection

        d = dict(src_dict)
        _images = d.pop("images", UNSET)
        images: ImageMediaCollection | Unset
        if isinstance(_images, Unset):
            images = UNSET
        else:
            images = ImageMediaCollection.from_dict(_images)

        _editorials = d.pop("editorials", UNSET)
        editorials: EditorialMediaCollection | Unset
        if isinstance(_editorials, Unset):
            editorials = UNSET
        else:
            editorials = EditorialMediaCollection.from_dict(_editorials)

        _ratings = d.pop("ratings", UNSET)
        ratings: RatingMediaCollection | Unset
        if isinstance(_ratings, Unset):
            ratings = UNSET
        else:
            ratings = RatingMediaCollection.from_dict(_ratings)

        media = cls(
            images=images,
            editorials=editorials,
            ratings=ratings,
        )

        media.additional_properties = d
        return media

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.notice_severity import NoticeSeverity
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.base_notice_detail import BaseNoticeDetail


T = TypeVar("T", bound="PedestrianNotice")


@_attrs_define
class PedestrianNotice:
    """A notice contains important notifications.

    Attributes:
        code (str): Extensible enum: `simplePolyline` `pedestrianOptionViolated` `violatedAvoidTunnel`
            `violatedAvoidDirtRoad` `violatedBlockedRoad` `...`
            Currently known codes (non-exhaustive: this list could be extended for new situations):

            | Code      | Description  | Severity |
            | --------- | ------- | ------- |
            | simplePolyline | An accurate polyline is not available for this section. The returned polyline has been
            generated from departure and arrival places | info |
            | pedestrianOptionViolated | This section violates the parameter `pedestrian[speed]` or
            `pedestrian[maxDistance]` | critical |
            | violatedAvoidTunnel | Route did not manage to avoid user preference | critical |
            | violatedAvoidDirtRoad | Route did not manage to avoid user preference | critical |
            | violatedBlockedRoad | Route uses roads that are blocked due to `avoid[areas]` or `avoid[segments]`. | critical
            |
             Example: violatedAvoidTunnel.
        title (str | Unset): Human-readable notice description. Example: Violated avoid tunnel.
        severity (NoticeSeverity | Unset): Describes the impact a notice has on the resource to which the notice is
            attached.
            * critical - The notice must not be ignored, even if the type of notice is not known to the user. Any associated
            resource (e.g., route section) must not be used without further evaluation.
            * info - The notice is for informative purposes, but does not affect usability of the route.
        details (list[BaseNoticeDetail] | Unset): Additional details about the notice
    """

    code: str
    title: str | Unset = UNSET
    severity: NoticeSeverity | Unset = UNSET
    details: list[BaseNoticeDetail] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        code = self.code

        title = self.title

        severity: str | Unset = UNSET
        if not isinstance(self.severity, Unset):
            severity = self.severity.value

        details: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.details, Unset):
            details = []
            for details_item_data in self.details:
                details_item = details_item_data.to_dict()
                details.append(details_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "code": code,
        })
        if title is not UNSET:
            field_dict["title"] = title
        if severity is not UNSET:
            field_dict["severity"] = severity
        if details is not UNSET:
            field_dict["details"] = details

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.base_notice_detail import BaseNoticeDetail

        d = dict(src_dict)
        code = d.pop("code")

        title = d.pop("title", UNSET)

        _severity = d.pop("severity", UNSET)
        severity: NoticeSeverity | Unset
        if isinstance(_severity, Unset):
            severity = UNSET
        else:
            severity = NoticeSeverity(_severity)

        _details = d.pop("details", UNSET)
        details: list[BaseNoticeDetail] | Unset = UNSET
        if _details is not UNSET:
            details = []
            for details_item_data in _details:
                details_item = BaseNoticeDetail.from_dict(details_item_data)

                details.append(details_item)

        pedestrian_notice = cls(
            code=code,
            title=title,
            severity=severity,
            details=details,
        )

        pedestrian_notice.additional_properties = d
        return pedestrian_notice

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ResponseRange")


@_attrs_define
class ResponseRange:
    """Range specified in terms of distance, travel time or energy consumption.

    Attributes:
        type_ (str): Specifies the type of range.

            Possible Values:
            - `distance` with units in meters
            - `time` with units in seconds
            - `consumption` with units in Watt-hours (Wh) for EV, or milliliters(ml) or grams(gms) for fuel-based vehicles,
            depending on `consumptionType`
        consumption_type (str | Unset): Specifies the consumption type. Returned only if range `type` is `consumption`.

            Possible Values:
              - `electric` : electric consumption type with units in Wh.
              - `diesel` : Diesel consumption type with units in ml.
              - `petrol` : Petrol/Gasoline consumption type with units in ml.
              - `lpg` : LPG consumption type with units in ml.
              - `cng` : CNG consumption type with units in gm.
        value (int | Unset): Range value. The unit is defined by the `type` parameter.
    """

    type_: str
    consumption_type: str | Unset = UNSET
    value: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        consumption_type = self.consumption_type

        value = self.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
        })
        if consumption_type is not UNSET:
            field_dict["consumptionType"] = consumption_type
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = d.pop("type")

        consumption_type = d.pop("consumptionType", UNSET)

        value = d.pop("value", UNSET)

        response_range = cls(
            type_=type_,
            consumption_type=consumption_type,
            value=value,
        )

        response_range.additional_properties = d
        return response_range

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EvAvailabilityEvseState(str, Enum):
    AVAILABLE = "AVAILABLE"
    OCCUPIED = "OCCUPIED"
    OFFLINE = "OFFLINE"
    OTHER = "OTHER"
    OUT_OF_SERVICE = "OUT_OF_SERVICE"
    RESERVED = "RESERVED"
    UNAVAILABLE = "UNAVAILABLE"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ErrorResponse")


@_attrs_define
class ErrorResponse:
    """
    Attributes:
        status (int): The HTTP status code
        title (str): Human-readable error description
        correlation_id (str): Auto-generated ID univocally identifying this request
        request_id (str): Request identifier provided by the user
        code (str | Unset): Error code
        cause (str | Unset): Human-readable explanation for the error
        action (str | Unset): Human-readable action for the user
    """

    status: int
    title: str
    correlation_id: str
    request_id: str
    code: str | Unset = UNSET
    cause: str | Unset = UNSET
    action: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        status = self.status

        title = self.title

        correlation_id = self.correlation_id

        request_id = self.request_id

        code = self.code

        cause = self.cause

        action = self.action

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "status": status,
            "title": title,
            "correlationId": correlation_id,
            "requestId": request_id,
        })
        if code is not UNSET:
            field_dict["code"] = code
        if cause is not UNSET:
            field_dict["cause"] = cause
        if action is not UNSET:
            field_dict["action"] = action

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        status = d.pop("status")

        title = d.pop("title")

        correlation_id = d.pop("correlationId")

        request_id = d.pop("requestId")

        code = d.pop("code", UNSET)

        cause = d.pop("cause", UNSET)

        action = d.pop("action", UNSET)

        error_response = cls(
            status=status,
            title=title,
            correlation_id=correlation_id,
            request_id=request_id,
            code=code,
            cause=cause,
            action=action,
        )

        error_response.additional_properties = d
        return error_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TurnActionAction(str, Enum):
    CONTINUEHIGHWAY = "continueHighway"
    ENTERHIGHWAY = "enterHighway"
    KEEP = "keep"
    RAMP = "ramp"
    ROUNDABOUTENTER = "roundaboutEnter"
    ROUNDABOUTPASS = "roundaboutPass"
    TURN = "turn"
    UTURN = "uTurn"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.geocode_result_item_address_block_type import (
    GeocodeResultItemAddressBlockType,
)
from ..models.geocode_result_item_administrative_area_type import (
    GeocodeResultItemAdministrativeAreaType,
)
from ..models.geocode_result_item_house_number_type import (
    GeocodeResultItemHouseNumberType,
)
from ..models.geocode_result_item_locality_type import GeocodeResultItemLocalityType
from ..models.geocode_result_item_result_type import GeocodeResultItemResultType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.address import Address
    from ..models.address_usage import AddressUsage
    from ..models.basic_access_point import BasicAccessPoint
    from ..models.category import Category
    from ..models.country_info import CountryInfo
    from ..models.display_response_coordinate import DisplayResponseCoordinate
    from ..models.map_reference_section import MapReferenceSection
    from ..models.map_view import MapView
    from ..models.navigation_attributes import NavigationAttributes
    from ..models.parsing import Parsing
    from ..models.postal_code_details_japan_post import PostalCodeDetailsJapanPost
    from ..models.postal_code_details_usps_zip import PostalCodeDetailsUspsZip
    from ..models.postal_code_details_usps_zip_plus_4 import (
        PostalCodeDetailsUspsZipPlus4,
    )
    from ..models.related_address import RelatedAddress
    from ..models.scoring import Scoring
    from ..models.secondary_unit_info import SecondaryUnitInfo
    from ..models.street_info import StreetInfo
    from ..models.time_zone_info import TimeZoneInfo
    from ..models.translations_geocode import TranslationsGeocode


T = TypeVar("T", bound="GeocodeResultItem")


@_attrs_define
class GeocodeResultItem:
    """
    Attributes:
        title (str): The localized display name of this result item.
        id (str): The unique identifier for the result item. This ID can be used for a Look Up by ID search as well.
        address (Address):
        political_view (str | Unset): ISO3 country code of the item political view (default for international). This
            response element is populated when the politicalView parameter is set in the query
        result_type (GeocodeResultItemResultType | Unset):
            Type of the result item.

            Note: `addressBlock` result item is either a block or subblock.

            `resultType` values can get added to the list without further notice.
        house_number_type (GeocodeResultItemHouseNumberType | Unset): Indicates the address type which affects the
            precision of the address and the coordinate accuracy.

            Description of supported values:

            - **RESTRICTED** `MPA`: A Micro Point Address represents a secondary address for a Point
              Address; for example, building, floor (level) and suite (unit).
              Micro Point Addresses can be used to enhance Point Address with
              greater address detail and higher coordinate accuracy. This result
              type is only returned by Lookup, Geocode or (Multi) Reverse Geocode endpoints when
              `with=MPA` parameter is provided.
            - `PA`: A Point Address represents an individual address as a point object.
              Point Addresses are coming from trusted sources. There is a
              high certainty that the address exists at that position. A
              Point Address result contains two types of coordinates. One is the
              access point (or navigation coordinates), which is the point to
              start or end a drive. The other point is the position or display
              point. This point varies per source and country. The point can be
              the rooftop point, a point close to the building entry, or a point
              close to the building, driveway or parking lot that belongs to the
              building.
            - `interpolated`: An interpolated address. These are approximate positions as a
              result of a linear interpolation based on address ranges. Address
              ranges, especially in the USA, are typical per block. For
              interpolated addresses, we cannot say with confidence that the
              address exists in reality. But the interpolation provides a good
              location approximation that brings people in most use cases close
              to the target location. The access point of an interpolated address
              result is calculated based on the address range and the road
              geometry. The position (display) point is pre-configured offset
              from the street geometry. Compared to Point Addresses, interpolated
              addresses are less accurate.
        address_usage (AddressUsage | Unset):
        address_block_type (GeocodeResultItemAddressBlockType | Unset):
            Type of the address block result if the `resultType` field is set to `addressBlock`.

            `addressBlockType` values can get added to the list without further notice.
        locality_type (GeocodeResultItemLocalityType | Unset):
            Type of the locality result if the `resultType` field is set to `locality`.

            `localityType` values can get added to the list without further notice.
        administrative_area_type (GeocodeResultItemAdministrativeAreaType | Unset):
            Type of the administrative area result if the `resultType` field is set to `administrativeArea`.

            `administrativeAreaType` values can get added to the list without further notice.
        postal_code_details (list[PostalCodeDetailsJapanPost | PostalCodeDetailsUspsZip | PostalCodeDetailsUspsZipPlus4]
            | Unset): Additional information for `postalCodePoint` results (only rendered if `show=postalCodeDetails` is
            provided and only in the countries which are using ZIP codes - USA, PRI, VIR, GUM, MNP, ASM and JPN)
        position (DisplayResponseCoordinate | Unset):
        access (list[BasicAccessPoint] | Unset): Coordinates of the response item on a HERE map navigable link (for
            example, for driving or walking).
        distance (int | Unset): The distance "as the crow flies" from the search center to this result item in meters.
            For example: "172039" Example: 172039.
        map_view (MapView | Unset):
        categories (list[Category] | Unset): The list of categories assigned to this place.
        food_types (list[Category] | Unset): The list of food types assigned to this place.
        house_number_fallback (bool | Unset): If true, indicates that the requested house number was corrected to match
            the nearest known house number. This field is visible only when the value is true.
        estimated_point_address (bool | Unset): If true, indicates that the coordinates of `position` and `access`
            points of the Point Address are
            estimated.
            This field is visible only for result items with resultType `houseNumber` and houseNumberType `PA` and
            only when the value is `true`
        time_zone (TimeZoneInfo | Unset):
        scoring (Scoring | Unset):
        parsing (Parsing | Unset):
        street_info (list[StreetInfo] | Unset): Street Details (only rendered if `show=streetInfo` is provided.)
        country_info (CountryInfo | Unset):
        translations (TranslationsGeocode | Unset):
        map_references (MapReferenceSection | Unset):
        related (list[RelatedAddress] | Unset): List of related objects
        navigation_attributes (NavigationAttributes | Unset):
        secondary_unit_info (list[SecondaryUnitInfo] | Unset): Secondary unit designator and unit value for each
            secondary unit recognized in the query in normalized country specific form (only returned if
            `show=secondaryUnitInfo` is provided.)
    """

    title: str
    id: str
    address: Address
    political_view: str | Unset = UNSET
    result_type: GeocodeResultItemResultType | Unset = UNSET
    house_number_type: GeocodeResultItemHouseNumberType | Unset = UNSET
    address_usage: AddressUsage | Unset = UNSET
    address_block_type: GeocodeResultItemAddressBlockType | Unset = UNSET
    locality_type: GeocodeResultItemLocalityType | Unset = UNSET
    administrative_area_type: GeocodeResultItemAdministrativeAreaType | Unset = UNSET
    postal_code_details: (
        list[
            PostalCodeDetailsJapanPost
            | PostalCodeDetailsUspsZip
            | PostalCodeDetailsUspsZipPlus4
        ]
        | Unset
    ) = UNSET
    position: DisplayResponseCoordinate | Unset = UNSET
    access: list[BasicAccessPoint] | Unset = UNSET
    distance: int | Unset = UNSET
    map_view: MapView | Unset = UNSET
    categories: list[Category] | Unset = UNSET
    food_types: list[Category] | Unset = UNSET
    house_number_fallback: bool | Unset = UNSET
    estimated_point_address: bool | Unset = UNSET
    time_zone: TimeZoneInfo | Unset = UNSET
    scoring: Scoring | Unset = UNSET
    parsing: Parsing | Unset = UNSET
    street_info: list[StreetInfo] | Unset = UNSET
    country_info: CountryInfo | Unset = UNSET
    translations: TranslationsGeocode | Unset = UNSET
    map_references: MapReferenceSection | Unset = UNSET
    related: list[RelatedAddress] | Unset = UNSET
    navigation_attributes: NavigationAttributes | Unset = UNSET
    secondary_unit_info: list[SecondaryUnitInfo] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.postal_code_details_japan_post import PostalCodeDetailsJapanPost
        from ..models.postal_code_details_usps_zip import PostalCodeDetailsUspsZip

        title = self.title

        id = self.id

        address = self.address.to_dict()

        political_view = self.political_view

        result_type: str | Unset = UNSET
        if not isinstance(self.result_type, Unset):
            result_type = self.result_type.value

        house_number_type: str | Unset = UNSET
        if not isinstance(self.house_number_type, Unset):
            house_number_type = self.house_number_type.value

        address_usage: dict[str, Any] | Unset = UNSET
        if not isinstance(self.address_usage, Unset):
            address_usage = self.address_usage.to_dict()

        address_block_type: str | Unset = UNSET
        if not isinstance(self.address_block_type, Unset):
            address_block_type = self.address_block_type.value

        locality_type: str | Unset = UNSET
        if not isinstance(self.locality_type, Unset):
            locality_type = self.locality_type.value

        administrative_area_type: str | Unset = UNSET
        if not isinstance(self.administrative_area_type, Unset):
            administrative_area_type = self.administrative_area_type.value

        postal_code_details: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.postal_code_details, Unset):
            postal_code_details = []
            for postal_code_details_item_data in self.postal_code_details:
                postal_code_details_item: dict[str, Any]
                if isinstance(
                    postal_code_details_item_data, PostalCodeDetailsJapanPost
                ):
                    postal_code_details_item = postal_code_details_item_data.to_dict()
                elif isinstance(
                    postal_code_details_item_data, PostalCodeDetailsUspsZip
                ):
                    postal_code_details_item = postal_code_details_item_data.to_dict()
                else:
                    postal_code_details_item = postal_code_details_item_data.to_dict()

                postal_code_details.append(postal_code_details_item)

        position: dict[str, Any] | Unset = UNSET
        if not isinstance(self.position, Unset):
            position = self.position.to_dict()

        access: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.access, Unset):
            access = []
            for access_item_data in self.access:
                access_item = access_item_data.to_dict()
                access.append(access_item)

        distance = self.distance

        map_view: dict[str, Any] | Unset = UNSET
        if not isinstance(self.map_view, Unset):
            map_view = self.map_view.to_dict()

        categories: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.categories, Unset):
            categories = []
            for categories_item_data in self.categories:
                categories_item = categories_item_data.to_dict()
                categories.append(categories_item)

        food_types: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.food_types, Unset):
            food_types = []
            for food_types_item_data in self.food_types:
                food_types_item = food_types_item_data.to_dict()
                food_types.append(food_types_item)

        house_number_fallback = self.house_number_fallback

        estimated_point_address = self.estimated_point_address

        time_zone: dict[str, Any] | Unset = UNSET
        if not isinstance(self.time_zone, Unset):
            time_zone = self.time_zone.to_dict()

        scoring: dict[str, Any] | Unset = UNSET
        if not isinstance(self.scoring, Unset):
            scoring = self.scoring.to_dict()

        parsing: dict[str, Any] | Unset = UNSET
        if not isinstance(self.parsing, Unset):
            parsing = self.parsing.to_dict()

        street_info: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.street_info, Unset):
            street_info = []
            for street_info_item_data in self.street_info:
                street_info_item = street_info_item_data.to_dict()
                street_info.append(street_info_item)

        country_info: dict[str, Any] | Unset = UNSET
        if not isinstance(self.country_info, Unset):
            country_info = self.country_info.to_dict()

        translations: dict[str, Any] | Unset = UNSET
        if not isinstance(self.translations, Unset):
            translations = self.translations.to_dict()

        map_references: dict[str, Any] | Unset = UNSET
        if not isinstance(self.map_references, Unset):
            map_references = self.map_references.to_dict()

        related: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.related, Unset):
            related = []
            for related_item_data in self.related:
                related_item = related_item_data.to_dict()
                related.append(related_item)

        navigation_attributes: dict[str, Any] | Unset = UNSET
        if not isinstance(self.navigation_attributes, Unset):
            navigation_attributes = self.navigation_attributes.to_dict()

        secondary_unit_info: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.secondary_unit_info, Unset):
            secondary_unit_info = []
            for secondary_unit_info_item_data in self.secondary_unit_info:
                secondary_unit_info_item = secondary_unit_info_item_data.to_dict()
                secondary_unit_info.append(secondary_unit_info_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "title": title,
            "id": id,
            "address": address,
        })
        if political_view is not UNSET:
            field_dict["politicalView"] = political_view
        if result_type is not UNSET:
            field_dict["resultType"] = result_type
        if house_number_type is not UNSET:
            field_dict["houseNumberType"] = house_number_type
        if address_usage is not UNSET:
            field_dict["addressUsage"] = address_usage
        if address_block_type is not UNSET:
            field_dict["addressBlockType"] = address_block_type
        if locality_type is not UNSET:
            field_dict["localityType"] = locality_type
        if administrative_area_type is not UNSET:
            field_dict["administrativeAreaType"] = administrative_area_type
        if postal_code_details is not UNSET:
            field_dict["postalCodeDetails"] = postal_code_details
        if position is not UNSET:
            field_dict["position"] = position
        if access is not UNSET:
            field_dict["access"] = access
        if distance is not UNSET:
            field_dict["distance"] = distance
        if map_view is not UNSET:
            field_dict["mapView"] = map_view
        if categories is not UNSET:
            field_dict["categories"] = categories
        if food_types is not UNSET:
            field_dict["foodTypes"] = food_types
        if house_number_fallback is not UNSET:
            field_dict["houseNumberFallback"] = house_number_fallback
        if estimated_point_address is not UNSET:
            field_dict["estimatedPointAddress"] = estimated_point_address
        if time_zone is not UNSET:
            field_dict["timeZone"] = time_zone
        if scoring is not UNSET:
            field_dict["scoring"] = scoring
        if parsing is not UNSET:
            field_dict["parsing"] = parsing
        if street_info is not UNSET:
            field_dict["streetInfo"] = street_info
        if country_info is not UNSET:
            field_dict["countryInfo"] = country_info
        if translations is not UNSET:
            field_dict["translations"] = translations
        if map_references is not UNSET:
            field_dict["mapReferences"] = map_references
        if related is not UNSET:
            field_dict["related"] = related
        if navigation_attributes is not UNSET:
            field_dict["navigationAttributes"] = navigation_attributes
        if secondary_unit_info is not UNSET:
            field_dict["secondaryUnitInfo"] = secondary_unit_info

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.address import Address
        from ..models.address_usage import AddressUsage
        from ..models.basic_access_point import BasicAccessPoint
        from ..models.category import Category
        from ..models.country_info import CountryInfo
        from ..models.display_response_coordinate import DisplayResponseCoordinate
        from ..models.map_reference_section import MapReferenceSection
        from ..models.map_view import MapView
        from ..models.navigation_attributes import NavigationAttributes
        from ..models.parsing import Parsing
        from ..models.postal_code_details_japan_post import PostalCodeDetailsJapanPost
        from ..models.postal_code_details_usps_zip import PostalCodeDetailsUspsZip
        from ..models.postal_code_details_usps_zip_plus_4 import (
            PostalCodeDetailsUspsZipPlus4,
        )
        from ..models.related_address import RelatedAddress
        from ..models.scoring import Scoring
        from ..models.secondary_unit_info import SecondaryUnitInfo
        from ..models.street_info import StreetInfo
        from ..models.time_zone_info import TimeZoneInfo
        from ..models.translations_geocode import TranslationsGeocode

        d = dict(src_dict)
        title = d.pop("title")

        id = d.pop("id")

        address = Address.from_dict(d.pop("address"))

        political_view = d.pop("politicalView", UNSET)

        _result_type = d.pop("resultType", UNSET)
        result_type: GeocodeResultItemResultType | Unset
        if isinstance(_result_type, Unset):
            result_type = UNSET
        else:
            result_type = GeocodeResultItemResultType(_result_type)

        _house_number_type = d.pop("houseNumberType", UNSET)
        house_number_type: GeocodeResultItemHouseNumberType | Unset
        if isinstance(_house_number_type, Unset):
            house_number_type = UNSET
        else:
            house_number_type = GeocodeResultItemHouseNumberType(_house_number_type)

        _address_usage = d.pop("addressUsage", UNSET)
        address_usage: AddressUsage | Unset
        if isinstance(_address_usage, Unset):
            address_usage = UNSET
        else:
            address_usage = AddressUsage.from_dict(_address_usage)

        _address_block_type = d.pop("addressBlockType", UNSET)
        address_block_type: GeocodeResultItemAddressBlockType | Unset
        if isinstance(_address_block_type, Unset):
            address_block_type = UNSET
        else:
            address_block_type = GeocodeResultItemAddressBlockType(_address_block_type)

        _locality_type = d.pop("localityType", UNSET)
        locality_type: GeocodeResultItemLocalityType | Unset
        if isinstance(_locality_type, Unset):
            locality_type = UNSET
        else:
            locality_type = GeocodeResultItemLocalityType(_locality_type)

        _administrative_area_type = d.pop("administrativeAreaType", UNSET)
        administrative_area_type: GeocodeResultItemAdministrativeAreaType | Unset
        if isinstance(_administrative_area_type, Unset):
            administrative_area_type = UNSET
        else:
            administrative_area_type = GeocodeResultItemAdministrativeAreaType(
                _administrative_area_type
            )

        _postal_code_details = d.pop("postalCodeDetails", UNSET)
        postal_code_details: (
            list[
                PostalCodeDetailsJapanPost
                | PostalCodeDetailsUspsZip
                | PostalCodeDetailsUspsZipPlus4
            ]
            | Unset
        ) = UNSET
        if _postal_code_details is not UNSET:
            postal_code_details = []
            for postal_code_details_item_data in _postal_code_details:

                def _parse_postal_code_details_item(
                    data: object,
                ) -> (
                    PostalCodeDetailsJapanPost
                    | PostalCodeDetailsUspsZip
                    | PostalCodeDetailsUspsZipPlus4
                ):
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        postal_code_details_item_type_0 = (
                            PostalCodeDetailsJapanPost.from_dict(data)
                        )

                        return postal_code_details_item_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        postal_code_details_item_type_1 = (
                            PostalCodeDetailsUspsZip.from_dict(data)
                        )

                        return postal_code_details_item_type_1
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    postal_code_details_item_type_2 = (
                        PostalCodeDetailsUspsZipPlus4.from_dict(data)
                    )

                    return postal_code_details_item_type_2

                postal_code_details_item = _parse_postal_code_details_item(
                    postal_code_details_item_data
                )

                postal_code_details.append(postal_code_details_item)

        _position = d.pop("position", UNSET)
        position: DisplayResponseCoordinate | Unset
        if isinstance(_position, Unset):
            position = UNSET
        else:
            position = DisplayResponseCoordinate.from_dict(_position)

        _access = d.pop("access", UNSET)
        access: list[BasicAccessPoint] | Unset = UNSET
        if _access is not UNSET:
            access = []
            for access_item_data in _access:
                access_item = BasicAccessPoint.from_dict(access_item_data)

                access.append(access_item)

        distance = d.pop("distance", UNSET)

        _map_view = d.pop("mapView", UNSET)
        map_view: MapView | Unset
        if isinstance(_map_view, Unset):
            map_view = UNSET
        else:
            map_view = MapView.from_dict(_map_view)

        _categories = d.pop("categories", UNSET)
        categories: list[Category] | Unset = UNSET
        if _categories is not UNSET:
            categories = []
            for categories_item_data in _categories:
                categories_item = Category.from_dict(categories_item_data)

                categories.append(categories_item)

        _food_types = d.pop("foodTypes", UNSET)
        food_types: list[Category] | Unset = UNSET
        if _food_types is not UNSET:
            food_types = []
            for food_types_item_data in _food_types:
                food_types_item = Category.from_dict(food_types_item_data)

                food_types.append(food_types_item)

        house_number_fallback = d.pop("houseNumberFallback", UNSET)

        estimated_point_address = d.pop("estimatedPointAddress", UNSET)

        _time_zone = d.pop("timeZone", UNSET)
        time_zone: TimeZoneInfo | Unset
        if isinstance(_time_zone, Unset):
            time_zone = UNSET
        else:
            time_zone = TimeZoneInfo.from_dict(_time_zone)

        _scoring = d.pop("scoring", UNSET)
        scoring: Scoring | Unset
        if isinstance(_scoring, Unset):
            scoring = UNSET
        else:
            scoring = Scoring.from_dict(_scoring)

        _parsing = d.pop("parsing", UNSET)
        parsing: Parsing | Unset
        if isinstance(_parsing, Unset):
            parsing = UNSET
        else:
            parsing = Parsing.from_dict(_parsing)

        _street_info = d.pop("streetInfo", UNSET)
        street_info: list[StreetInfo] | Unset = UNSET
        if _street_info is not UNSET:
            street_info = []
            for street_info_item_data in _street_info:
                street_info_item = StreetInfo.from_dict(street_info_item_data)

                street_info.append(street_info_item)

        _country_info = d.pop("countryInfo", UNSET)
        country_info: CountryInfo | Unset
        if isinstance(_country_info, Unset):
            country_info = UNSET
        else:
            country_info = CountryInfo.from_dict(_country_info)

        _translations = d.pop("translations", UNSET)
        translations: TranslationsGeocode | Unset
        if isinstance(_translations, Unset):
            translations = UNSET
        else:
            translations = TranslationsGeocode.from_dict(_translations)

        _map_references = d.pop("mapReferences", UNSET)
        map_references: MapReferenceSection | Unset
        if isinstance(_map_references, Unset):
            map_references = UNSET
        else:
            map_references = MapReferenceSection.from_dict(_map_references)

        _related = d.pop("related", UNSET)
        related: list[RelatedAddress] | Unset = UNSET
        if _related is not UNSET:
            related = []
            for related_item_data in _related:
                related_item = RelatedAddress.from_dict(related_item_data)

                related.append(related_item)

        _navigation_attributes = d.pop("navigationAttributes", UNSET)
        navigation_attributes: NavigationAttributes | Unset
        if isinstance(_navigation_attributes, Unset):
            navigation_attributes = UNSET
        else:
            navigation_attributes = NavigationAttributes.from_dict(
                _navigation_attributes
            )

        _secondary_unit_info = d.pop("secondaryUnitInfo", UNSET)
        secondary_unit_info: list[SecondaryUnitInfo] | Unset = UNSET
        if _secondary_unit_info is not UNSET:
            secondary_unit_info = []
            for secondary_unit_info_item_data in _secondary_unit_info:
                secondary_unit_info_item = SecondaryUnitInfo.from_dict(
                    secondary_unit_info_item_data
                )

                secondary_unit_info.append(secondary_unit_info_item)

        geocode_result_item = cls(
            title=title,
            id=id,
            address=address,
            political_view=political_view,
            result_type=result_type,
            house_number_type=house_number_type,
            address_usage=address_usage,
            address_block_type=address_block_type,
            locality_type=locality_type,
            administrative_area_type=administrative_area_type,
            postal_code_details=postal_code_details,
            position=position,
            access=access,
            distance=distance,
            map_view=map_view,
            categories=categories,
            food_types=food_types,
            house_number_fallback=house_number_fallback,
            estimated_point_address=estimated_point_address,
            time_zone=time_zone,
            scoring=scoring,
            parsing=parsing,
            street_info=street_info,
            country_info=country_info,
            translations=translations,
            map_references=map_references,
            related=related,
            navigation_attributes=navigation_attributes,
            secondary_unit_info=secondary_unit_info,
        )

        geocode_result_item.additional_properties = d
        return geocode_result_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.onebox_search_result_item_address_block_type import (
    OneboxSearchResultItemAddressBlockType,
)
from ..models.onebox_search_result_item_administrative_area_type import (
    OneboxSearchResultItemAdministrativeAreaType,
)
from ..models.onebox_search_result_item_house_number_type import (
    OneboxSearchResultItemHouseNumberType,
)
from ..models.onebox_search_result_item_locality_type import (
    OneboxSearchResultItemLocalityType,
)
from ..models.onebox_search_result_item_result_type import (
    OneboxSearchResultItemResultType,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.access_restriction_attributes import AccessRestrictionAttributes
    from ..models.address import Address
    from ..models.category import Category
    from ..models.chain import Chain
    from ..models.contact_information import ContactInformation
    from ..models.display_response_coordinate import DisplayResponseCoordinate
    from ..models.extended_access_point import ExtendedAccessPoint
    from ..models.extended_attribute import ExtendedAttribute
    from ..models.map_view import MapView
    from ..models.media import Media
    from ..models.opening_hours import OpeningHours
    from ..models.phonemes_section import PhonemesSection
    from ..models.street_info import StreetInfo
    from ..models.supplier_reference import SupplierReference
    from ..models.time_zone_info import TimeZoneInfo


T = TypeVar("T", bound="OneboxSearchResultItem")


@_attrs_define
class OneboxSearchResultItem:
    """
    Attributes:
        title (str): The localized display name of this result item.
        id (str): The unique identifier for the result item. This ID can be used for a Look Up by ID search as well.
        address (Address):
        political_view (str | Unset): ISO3 country code of the item political view (default for international). This
            response element is populated when the politicalView parameter is set in the query
        ontology_id (str | Unset): Related ontology ID
        result_type (OneboxSearchResultItemResultType | Unset):
            Type of the result item.

            Note: `addressBlock` result item is either a block or subblock.

            `resultType` values can get added to the list without further notice.
        house_number_type (OneboxSearchResultItemHouseNumberType | Unset): Indicates the address type which affects the
            precision of the address and the coordinate accuracy.

            Description of supported values:

            - **RESTRICTED** `MPA`: A Micro Point Address represents a secondary address for a Point
              Address; for example, building, floor (level) and suite (unit).
              Micro Point Addresses can be used to enhance Point Address with
              greater address detail and higher coordinate accuracy. This result
              type is only returned by Lookup, Geocode or (Multi) Reverse Geocode endpoints when
              `with=MPA` parameter is provided.
            - `PA`: A Point Address represents an individual address as a point object.
              Point Addresses are coming from trusted sources. There is a
              high certainty that the address exists at that position. A
              Point Address result contains two types of coordinates. One is the
              access point (or navigation coordinates), which is the point to
              start or end a drive. The other point is the position or display
              point. This point varies per source and country. The point can be
              the rooftop point, a point close to the building entry, or a point
              close to the building, driveway or parking lot that belongs to the
              building.
            - `interpolated`: An interpolated address. These are approximate positions as a
              result of a linear interpolation based on address ranges. Address
              ranges, especially in the USA, are typical per block. For
              interpolated addresses, we cannot say with confidence that the
              address exists in reality. But the interpolation provides a good
              location approximation that brings people in most use cases close
              to the target location. The access point of an interpolated address
              result is calculated based on the address range and the road
              geometry. The position (display) point is pre-configured offset
              from the street geometry. Compared to Point Addresses, interpolated
              addresses are less accurate.
        address_block_type (OneboxSearchResultItemAddressBlockType | Unset):
            Type of the address block result if the `resultType` field is set to `addressBlock`.

            `addressBlockType` values can get added to the list without further notice.
        locality_type (OneboxSearchResultItemLocalityType | Unset):
            Type of the locality result if the `resultType` field is set to `locality`.

            `localityType` values can get added to the list without further notice.
        administrative_area_type (OneboxSearchResultItemAdministrativeAreaType | Unset):
            Type of the administrative area result if the `resultType` field is set to `administrativeArea`.

            `administrativeAreaType` values can get added to the list without further notice.
        position (DisplayResponseCoordinate | Unset):
        access (list[ExtendedAccessPoint] | Unset): Coordinates of the response item on a HERE map navigable link (for
            example, for driving or walking).

            When a response item is of type `"place"`, it can contain more than one access object.
            Each access object contains a position (`lat`, `lng`), with potentially a type (field `type`),
            a textual label (field `label`) and a `primary` boolean field set to true for the main access to the place.

            The first object in the `access` array is the primary access to the place.

            Response items of other types than `"place"` have a maximum of one access position.
        drive_through (bool | Unset): Set to `true` if the related business allows customers to purchase products
            (or use the service provided by the business) without leaving their cars.
        distance (int | Unset): The distance "as the crow flies" from the search center to this result item in meters.
            For example: "172039".

            When searching along a route this is the distance along the route plus the distance from the route polyline to
            this result item. Example: 172039.
        excursion_distance (int | Unset): Two times the distance from the polyline to this result item in meters while
            searching along the route
        map_view (MapView | Unset):
        categories (list[Category] | Unset): The list of categories assigned to this place.
        chains (list[Chain] | Unset): The list of chains assigned to this place.
        references (list[SupplierReference] | Unset): The list of supplier references available for this place.
        food_types (list[Category] | Unset): The list of food types assigned to this place.
        house_number_fallback (bool | Unset): If true, indicates that the requested house number was corrected to match
            the nearest known house number. This field is visible only when the value is true.
        contacts (list[ContactInformation] | Unset): Contact information like phone, email, WWW.
        opening_hours (list[OpeningHours] | Unset): A list of hours during which the place is open for business.
            This field is optional: When it is not present, it means that we are lacking data about the place opening hours.
            Days without opening hours have to be considered as closed.
        time_zone (TimeZoneInfo | Unset):
        extended (ExtendedAttribute | Unset):
        phonemes (PhonemesSection | Unset):
        media (Media | Unset):
        street_info (list[StreetInfo] | Unset): Street Details (only rendered if `show=streetInfo` is provided.)
        access_restrictions (list[AccessRestrictionAttributes] | Unset): A list of access restrictions related to a
            `place` result item.

            A place response item has a restricted access when the place is not publicly accessible without a specific
            authorization, membership or customer relationship.
            Examples: the place is for customers or visitors only.

            This field is optional: When it is not present, it means that the place response item has no known access
            restriction. Example: [{'categories': [{'id': '800-8500-0177'}], 'restricted': True}].
    """

    title: str
    id: str
    address: Address
    political_view: str | Unset = UNSET
    ontology_id: str | Unset = UNSET
    result_type: OneboxSearchResultItemResultType | Unset = UNSET
    house_number_type: OneboxSearchResultItemHouseNumberType | Unset = UNSET
    address_block_type: OneboxSearchResultItemAddressBlockType | Unset = UNSET
    locality_type: OneboxSearchResultItemLocalityType | Unset = UNSET
    administrative_area_type: OneboxSearchResultItemAdministrativeAreaType | Unset = (
        UNSET
    )
    position: DisplayResponseCoordinate | Unset = UNSET
    access: list[ExtendedAccessPoint] | Unset = UNSET
    drive_through: bool | Unset = UNSET
    distance: int | Unset = UNSET
    excursion_distance: int | Unset = UNSET
    map_view: MapView | Unset = UNSET
    categories: list[Category] | Unset = UNSET
    chains: list[Chain] | Unset = UNSET
    references: list[SupplierReference] | Unset = UNSET
    food_types: list[Category] | Unset = UNSET
    house_number_fallback: bool | Unset = UNSET
    contacts: list[ContactInformation] | Unset = UNSET
    opening_hours: list[OpeningHours] | Unset = UNSET
    time_zone: TimeZoneInfo | Unset = UNSET
    extended: ExtendedAttribute | Unset = UNSET
    phonemes: PhonemesSection | Unset = UNSET
    media: Media | Unset = UNSET
    street_info: list[StreetInfo] | Unset = UNSET
    access_restrictions: list[AccessRestrictionAttributes] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        title = self.title

        id = self.id

        address = self.address.to_dict()

        political_view = self.political_view

        ontology_id = self.ontology_id

        result_type: str | Unset = UNSET
        if not isinstance(self.result_type, Unset):
            result_type = self.result_type.value

        house_number_type: str | Unset = UNSET
        if not isinstance(self.house_number_type, Unset):
            house_number_type = self.house_number_type.value

        address_block_type: str | Unset = UNSET
        if not isinstance(self.address_block_type, Unset):
            address_block_type = self.address_block_type.value

        locality_type: str | Unset = UNSET
        if not isinstance(self.locality_type, Unset):
            locality_type = self.locality_type.value

        administrative_area_type: str | Unset = UNSET
        if not isinstance(self.administrative_area_type, Unset):
            administrative_area_type = self.administrative_area_type.value

        position: dict[str, Any] | Unset = UNSET
        if not isinstance(self.position, Unset):
            position = self.position.to_dict()

        access: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.access, Unset):
            access = []
            for access_item_data in self.access:
                access_item = access_item_data.to_dict()
                access.append(access_item)

        drive_through = self.drive_through

        distance = self.distance

        excursion_distance = self.excursion_distance

        map_view: dict[str, Any] | Unset = UNSET
        if not isinstance(self.map_view, Unset):
            map_view = self.map_view.to_dict()

        categories: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.categories, Unset):
            categories = []
            for categories_item_data in self.categories:
                categories_item = categories_item_data.to_dict()
                categories.append(categories_item)

        chains: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.chains, Unset):
            chains = []
            for chains_item_data in self.chains:
                chains_item = chains_item_data.to_dict()
                chains.append(chains_item)

        references: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.references, Unset):
            references = []
            for references_item_data in self.references:
                references_item = references_item_data.to_dict()
                references.append(references_item)

        food_types: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.food_types, Unset):
            food_types = []
            for food_types_item_data in self.food_types:
                food_types_item = food_types_item_data.to_dict()
                food_types.append(food_types_item)

        house_number_fallback = self.house_number_fallback

        contacts: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.contacts, Unset):
            contacts = []
            for contacts_item_data in self.contacts:
                contacts_item = contacts_item_data.to_dict()
                contacts.append(contacts_item)

        opening_hours: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.opening_hours, Unset):
            opening_hours = []
            for opening_hours_item_data in self.opening_hours:
                opening_hours_item = opening_hours_item_data.to_dict()
                opening_hours.append(opening_hours_item)

        time_zone: dict[str, Any] | Unset = UNSET
        if not isinstance(self.time_zone, Unset):
            time_zone = self.time_zone.to_dict()

        extended: dict[str, Any] | Unset = UNSET
        if not isinstance(self.extended, Unset):
            extended = self.extended.to_dict()

        phonemes: dict[str, Any] | Unset = UNSET
        if not isinstance(self.phonemes, Unset):
            phonemes = self.phonemes.to_dict()

        media: dict[str, Any] | Unset = UNSET
        if not isinstance(self.media, Unset):
            media = self.media.to_dict()

        street_info: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.street_info, Unset):
            street_info = []
            for street_info_item_data in self.street_info:
                street_info_item = street_info_item_data.to_dict()
                street_info.append(street_info_item)

        access_restrictions: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.access_restrictions, Unset):
            access_restrictions = []
            for access_restrictions_item_data in self.access_restrictions:
                access_restrictions_item = access_restrictions_item_data.to_dict()
                access_restrictions.append(access_restrictions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "title": title,
            "id": id,
            "address": address,
        })
        if political_view is not UNSET:
            field_dict["politicalView"] = political_view
        if ontology_id is not UNSET:
            field_dict["ontologyId"] = ontology_id
        if result_type is not UNSET:
            field_dict["resultType"] = result_type
        if house_number_type is not UNSET:
            field_dict["houseNumberType"] = house_number_type
        if address_block_type is not UNSET:
            field_dict["addressBlockType"] = address_block_type
        if locality_type is not UNSET:
            field_dict["localityType"] = locality_type
        if administrative_area_type is not UNSET:
            field_dict["administrativeAreaType"] = administrative_area_type
        if position is not UNSET:
            field_dict["position"] = position
        if access is not UNSET:
            field_dict["access"] = access
        if drive_through is not UNSET:
            field_dict["driveThrough"] = drive_through
        if distance is not UNSET:
            field_dict["distance"] = distance
        if excursion_distance is not UNSET:
            field_dict["excursionDistance"] = excursion_distance
        if map_view is not UNSET:
            field_dict["mapView"] = map_view
        if categories is not UNSET:
            field_dict["categories"] = categories
        if chains is not UNSET:
            field_dict["chains"] = chains
        if references is not UNSET:
            field_dict["references"] = references
        if food_types is not UNSET:
            field_dict["foodTypes"] = food_types
        if house_number_fallback is not UNSET:
            field_dict["houseNumberFallback"] = house_number_fallback
        if contacts is not UNSET:
            field_dict["contacts"] = contacts
        if opening_hours is not UNSET:
            field_dict["openingHours"] = opening_hours
        if time_zone is not UNSET:
            field_dict["timeZone"] = time_zone
        if extended is not UNSET:
            field_dict["extended"] = extended
        if phonemes is not UNSET:
            field_dict["phonemes"] = phonemes
        if media is not UNSET:
            field_dict["media"] = media
        if street_info is not UNSET:
            field_dict["streetInfo"] = street_info
        if access_restrictions is not UNSET:
            field_dict["accessRestrictions"] = access_restrictions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.access_restriction_attributes import AccessRestrictionAttributes
        from ..models.address import Address
        from ..models.category import Category
        from ..models.chain import Chain
        from ..models.contact_information import ContactInformation
        from ..models.display_response_coordinate import DisplayResponseCoordinate
        from ..models.extended_access_point import ExtendedAccessPoint
        from ..models.extended_attribute import ExtendedAttribute
        from ..models.map_view import MapView
        from ..models.media import Media
        from ..models.opening_hours import OpeningHours
        from ..models.phonemes_section import PhonemesSection
        from ..models.street_info import StreetInfo
        from ..models.supplier_reference import SupplierReference
        from ..models.time_zone_info import TimeZoneInfo

        d = dict(src_dict)
        title = d.pop("title")

        id = d.pop("id")

        address = Address.from_dict(d.pop("address"))

        political_view = d.pop("politicalView", UNSET)

        ontology_id = d.pop("ontologyId", UNSET)

        _result_type = d.pop("resultType", UNSET)
        result_type: OneboxSearchResultItemResultType | Unset
        if isinstance(_result_type, Unset):
            result_type = UNSET
        else:
            result_type = OneboxSearchResultItemResultType(_result_type)

        _house_number_type = d.pop("houseNumberType", UNSET)
        house_number_type: OneboxSearchResultItemHouseNumberType | Unset
        if isinstance(_house_number_type, Unset):
            house_number_type = UNSET
        else:
            house_number_type = OneboxSearchResultItemHouseNumberType(
                _house_number_type
            )

        _address_block_type = d.pop("addressBlockType", UNSET)
        address_block_type: OneboxSearchResultItemAddressBlockType | Unset
        if isinstance(_address_block_type, Unset):
            address_block_type = UNSET
        else:
            address_block_type = OneboxSearchResultItemAddressBlockType(
                _address_block_type
            )

        _locality_type = d.pop("localityType", UNSET)
        locality_type: OneboxSearchResultItemLocalityType | Unset
        if isinstance(_locality_type, Unset):
            locality_type = UNSET
        else:
            locality_type = OneboxSearchResultItemLocalityType(_locality_type)

        _administrative_area_type = d.pop("administrativeAreaType", UNSET)
        administrative_area_type: OneboxSearchResultItemAdministrativeAreaType | Unset
        if isinstance(_administrative_area_type, Unset):
            administrative_area_type = UNSET
        else:
            administrative_area_type = OneboxSearchResultItemAdministrativeAreaType(
                _administrative_area_type
            )

        _position = d.pop("position", UNSET)
        position: DisplayResponseCoordinate | Unset
        if isinstance(_position, Unset):
            position = UNSET
        else:
            position = DisplayResponseCoordinate.from_dict(_position)

        _access = d.pop("access", UNSET)
        access: list[ExtendedAccessPoint] | Unset = UNSET
        if _access is not UNSET:
            access = []
            for access_item_data in _access:
                access_item = ExtendedAccessPoint.from_dict(access_item_data)

                access.append(access_item)

        drive_through = d.pop("driveThrough", UNSET)

        distance = d.pop("distance", UNSET)

        excursion_distance = d.pop("excursionDistance", UNSET)

        _map_view = d.pop("mapView", UNSET)
        map_view: MapView | Unset
        if isinstance(_map_view, Unset):
            map_view = UNSET
        else:
            map_view = MapView.from_dict(_map_view)

        _categories = d.pop("categories", UNSET)
        categories: list[Category] | Unset = UNSET
        if _categories is not UNSET:
            categories = []
            for categories_item_data in _categories:
                categories_item = Category.from_dict(categories_item_data)

                categories.append(categories_item)

        _chains = d.pop("chains", UNSET)
        chains: list[Chain] | Unset = UNSET
        if _chains is not UNSET:
            chains = []
            for chains_item_data in _chains:
                chains_item = Chain.from_dict(chains_item_data)

                chains.append(chains_item)

        _references = d.pop("references", UNSET)
        references: list[SupplierReference] | Unset = UNSET
        if _references is not UNSET:
            references = []
            for references_item_data in _references:
                references_item = SupplierReference.from_dict(references_item_data)

                references.append(references_item)

        _food_types = d.pop("foodTypes", UNSET)
        food_types: list[Category] | Unset = UNSET
        if _food_types is not UNSET:
            food_types = []
            for food_types_item_data in _food_types:
                food_types_item = Category.from_dict(food_types_item_data)

                food_types.append(food_types_item)

        house_number_fallback = d.pop("houseNumberFallback", UNSET)

        _contacts = d.pop("contacts", UNSET)
        contacts: list[ContactInformation] | Unset = UNSET
        if _contacts is not UNSET:
            contacts = []
            for contacts_item_data in _contacts:
                contacts_item = ContactInformation.from_dict(contacts_item_data)

                contacts.append(contacts_item)

        _opening_hours = d.pop("openingHours", UNSET)
        opening_hours: list[OpeningHours] | Unset = UNSET
        if _opening_hours is not UNSET:
            opening_hours = []
            for opening_hours_item_data in _opening_hours:
                opening_hours_item = OpeningHours.from_dict(opening_hours_item_data)

                opening_hours.append(opening_hours_item)

        _time_zone = d.pop("timeZone", UNSET)
        time_zone: TimeZoneInfo | Unset
        if isinstance(_time_zone, Unset):
            time_zone = UNSET
        else:
            time_zone = TimeZoneInfo.from_dict(_time_zone)

        _extended = d.pop("extended", UNSET)
        extended: ExtendedAttribute | Unset
        if isinstance(_extended, Unset):
            extended = UNSET
        else:
            extended = ExtendedAttribute.from_dict(_extended)

        _phonemes = d.pop("phonemes", UNSET)
        phonemes: PhonemesSection | Unset
        if isinstance(_phonemes, Unset):
            phonemes = UNSET
        else:
            phonemes = PhonemesSection.from_dict(_phonemes)

        _media = d.pop("media", UNSET)
        media: Media | Unset
        if isinstance(_media, Unset):
            media = UNSET
        else:
            media = Media.from_dict(_media)

        _street_info = d.pop("streetInfo", UNSET)
        street_info: list[StreetInfo] | Unset = UNSET
        if _street_info is not UNSET:
            street_info = []
            for street_info_item_data in _street_info:
                street_info_item = StreetInfo.from_dict(street_info_item_data)

                street_info.append(street_info_item)

        _access_restrictions = d.pop("accessRestrictions", UNSET)
        access_restrictions: list[AccessRestrictionAttributes] | Unset = UNSET
        if _access_restrictions is not UNSET:
            access_restrictions = []
            for access_restrictions_item_data in _access_restrictions:
                access_restrictions_item = AccessRestrictionAttributes.from_dict(
                    access_restrictions_item_data
                )

                access_restrictions.append(access_restrictions_item)

        onebox_search_result_item = cls(
            title=title,
            id=id,
            address=address,
            political_view=political_view,
            ontology_id=ontology_id,
            result_type=result_type,
            house_number_type=house_number_type,
            address_block_type=address_block_type,
            locality_type=locality_type,
            administrative_area_type=administrative_area_type,
            position=position,
            access=access,
            drive_through=drive_through,
            distance=distance,
            excursion_distance=excursion_distance,
            map_view=map_view,
            categories=categories,
            chains=chains,
            references=references,
            food_types=food_types,
            house_number_fallback=house_number_fallback,
            contacts=contacts,
            opening_hours=opening_hours,
            time_zone=time_zone,
            extended=extended,
            phonemes=phonemes,
            media=media,
            street_info=street_info,
            access_restrictions=access_restrictions,
        )

        onebox_search_result_item.additional_properties = d
        return onebox_search_result_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
